/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	MoreATSUnicode.cp	-	Source file
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
------------------------------------------------------------------------------*/

#include "MoreATSUnicodeInternal.h"

#include <ATSLayoutTypes.h>
#include <FixMath.h>
#include <Script.h>
#include <stdlib.h>

#include "MoreATSUnicode.h"


/* Getters and Setters */

OSStatus
atsuSetBooleanAttribute( ATSUStyle iStyle, Boolean iSetting, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iSetting);
	ATSUAttributeValuePtr	thePtr = &iSetting;

	return ATSUSetAttributes( iStyle, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetFixedAttribute( ATSUStyle iStyle, Fixed iValue, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iValue);
	ATSUAttributeValuePtr	thePtr = &iValue;

	return ATSUSetAttributes( iStyle, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetFractAttribute( ATSUStyle iStyle, Fract iValue, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iValue);
	ATSUAttributeValuePtr	thePtr = &iValue;

	return ATSUSetAttributes( iStyle, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetFont( ATSUStyle iStyle, ATSUFontID fontID )
{
	ATSUAttributeTag		theTag = kATSUFontTag;
	ByteCount				theSize = sizeof(fontID);
	ATSUAttributeValuePtr	thePtr = &fontID;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetColor( ATSUStyle iStyle, RGBColor color )
{
	ATSUAttributeTag		theTag = kATSUColorTag;
	ByteCount				theSize = sizeof(RGBColor);
	ATSUAttributeValuePtr	thePtr = &color;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetLanguage( ATSUStyle iStyle, RegionCode language )
{
	ATSUAttributeTag		theTag = kATSULanguageTag;
	ByteCount				theSize = sizeof(language);
	ATSUAttributeValuePtr	thePtr = &language;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetVerticalCharacter( ATSUStyle iStyle, ATSUVerticalCharacterType vertical )
{
	ATSUAttributeTag		theTag = kATSUVerticalCharacterTag;
	ByteCount				theSize = sizeof(vertical);
	ATSUAttributeValuePtr	thePtr = &vertical;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetBaselineClass( ATSUStyle iStyle, BslnBaselineClass baselineClass )
{
	ATSUAttributeTag		theTag = kATSUBaselineClassTag;
	ByteCount				theSize = sizeof(baselineClass);
	ATSUAttributeValuePtr	thePtr = &baselineClass;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetPriorityJustOverride( ATSUStyle iStyle, ATSJustPriorityWidthDeltaOverrides overrides )
{
	ATSUAttributeTag		theTag = kATSUPriorityJustOverrideTag;
	ByteCount				theSize = sizeof(ATSJustPriorityWidthDeltaOverrides);
	ATSUAttributeValuePtr	thePtr = &overrides;
	return ATSUSetAttributes( iStyle, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetBooleanLayoutControl( ATSUTextLayout iLayout, Boolean iValue, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iValue);
	ATSUAttributeValuePtr	thePtr = &iValue;

	return ATSUSetLayoutControls( iLayout, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetFixedLayoutControl( ATSUTextLayout iLayout, Fixed iValue, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iValue);
	ATSUAttributeValuePtr	thePtr = &iValue;

	return ATSUSetLayoutControls( iLayout, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetFractLayoutControl( ATSUTextLayout iLayout, Fract iValue, ATSUAttributeTag iTag )
{
	ByteCount				theSize = sizeof(iValue);
	ATSUAttributeValuePtr	thePtr = &iValue;

	return ATSUSetLayoutControls( iLayout, 1, &iTag, &theSize, &thePtr );
}


OSStatus
atsuSetLineBaselineValues( ATSUTextLayout iLayout, BslnBaselineRecord iValues )
{
	ATSUAttributeTag		theTag = kATSULineBaselineValuesTag;
	ByteCount				theSize = sizeof(BslnBaselineRecord);
	ATSUAttributeValuePtr	thePtr = &iValues;

	return ATSUSetLayoutControls( iLayout, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuSetLineLayoutOptions( ATSUTextLayout iLayout, UInt32 iOptions )
{
	ATSUAttributeTag		theTag = kATSULineLayoutOptionsTag;
	ByteCount				theSize = sizeof(iOptions);
	ATSUAttributeValuePtr	thePtr = &iOptions;

	return ATSUSetLayoutControls( iLayout, 1, &theTag, &theSize, &thePtr );
}


OSStatus
atsuGetOneAttribute( ATSUStyle 				iStyle,
					 ATSUAttributeTag 		iTag,
					 ByteCount 				iExpectedValueSize,
					 ATSUAttributeValuePtr 	oValue )
{
	// Filters out kATSUNotSetErr
	OSStatus status = ATSUGetAttribute( iStyle, iTag, iExpectedValueSize, oValue, NULL );
	return (status == kATSUNotSetErr) ? noErr : status;
}


OSStatus
atsuGetOneLayoutControl( ATSUTextLayout 		iLayout,
						 ATSUAttributeTag 		iTag,
						 ByteCount 				iExpectedValueSize,
						 ATSUAttributeValuePtr 	oValue )
{
	// Filters out kATSUNotSetErr
	OSStatus status = ATSUGetLayoutControl( iLayout, iTag, iExpectedValueSize, oValue, NULL );
	return (status == kATSUNotSetErr) ? noErr : status;
}



OSStatus
atsuSetStyleFromGrafPtr( ATSUStyle iStyle, GrafPtr iGrafPtr, Boolean clearExistingAttributes )
{
	OSStatus	status = noErr;
	GrafPtr		savedPort;
	ATSUAttributeTag		theTags[] = {	kATSUFontTag,
											kATSUSizeTag,
										//	kATSUColorTag,
											kATSUQDBoldfaceTag,
											kATSUQDItalicTag,
											kATSUQDUnderlineTag,
											kATSUQDCondensedTag,
											kATSUQDExtendedTag };
	ByteCount				theSizes[] = {	sizeof(ATSUFontID),
											sizeof(Fixed),
										//	sizeof(RGBColor),
											sizeof(Boolean),
											sizeof(Boolean),
											sizeof(Boolean),
											sizeof(Boolean),
											sizeof(Boolean) };
	ATSUFontID			atsuFont;
	Fixed				atsuSize;
	RGBColor			textColor;
	Boolean				isBold, isItalic, isUnderline, isCondensed, isExtended;
	ATSUAttributeValuePtr	theValues[] = {	&atsuFont,
											&atsuSize,
										//	&textColor,
											&isBold,
											&isItalic,
											&isUnderline,
											&isCondensed,
											&isExtended };
	short		txFont, txSize;
	Style		txFace;
	
	
	GetPort( &savedPort );
	
	if ( iGrafPtr == NULL ) 
		iGrafPtr = savedPort;
	else
		SetPort( iGrafPtr );
		
	if ( clearExistingAttributes ) 
		status = ATSUClearStyle( iStyle );
	require( status == noErr, EXIT );
	
	txFont = iGrafPtr->txFont;
	txSize = iGrafPtr->txSize;
	txFace = iGrafPtr->txFace;
	
	status = ATSUFONDtoFontID( txFont, txFace, &atsuFont );
	require( status == noErr, EXIT );
	
	if ( txSize == 0 )
		txSize = (short) ( GetScriptVariable( FontToScript( txFont ), smScriptPrefFondSize ) & 0xFFFFU );	// this would already be set correctly in a brand-new style
	atsuSize = Long2Fix( txSize );
	
	isBold = ( txFace & bold ) != 0;
	isItalic = ( txFace & italic ) != 0;
	isUnderline = ( txFace & underline ) != 0;
	isCondensed = ( txFace & condense ) != 0;
	isExtended = ( txFace & extend ) != 0;
	
	GetForeColor( &textColor );
	
	status = ATSUSetAttributes( iStyle, sizeof(theTags) / sizeof(ATSUAttributeTag), theTags, theSizes, theValues );
	
EXIT:	
	SetPort( savedPort );

	return status;
}


// NB: even though the coordinate system is left-handed, positive rotations
// are counterclockwise, so we use the inverse transformation of what you'd expect.
static void RotateAndTranslate( Fixed x, Fixed y, Fixed rotateCos, Fixed rotateSin, Fixed x0, Fixed y0, atsuFixedPt *pt )
{
	pt->x =  FixMul(x, rotateCos) + FixMul(y, rotateSin) + x0;
	pt->y =  FixMul(y, rotateCos) - FixMul(x, rotateSin) + y0;
}



#if !USE_CONSY_ROTATION

OSStatus
atsuMeasureRotatedTextImage( ATSUTextLayout iTextLayout, 
							UniCharArrayOffset iLineOffset,
							UniCharCount iLineLength,
							ATSUTextMeasurement iLocationX,
							ATSUTextMeasurement iLocationY,
							atsuFixedPt *p1,
							atsuFixedPt *p2,
							atsuFixedPt *p3,
							atsuFixedPt *p4 )
{
/* 
   The simple approach is to simply set the line rotation to zero, get the image bounds
   and set it back again. The drawback to doing this is that it will potentially blow
   away cached layout information for the *entire* layout.
 */
 
	OSStatus 				status = noErr;
	Fixed 					lineRotation = 0;
	Rect imageRect;
	
	// Get the current rotation.
	status = atsuGetLineRotation( iTextLayout, &lineRotation );
	require( status == noErr, EXIT );
	
	// If the line rotation is a multiple of 90 degrees, the standard routine gives us the right answer.
	if ( lineRotation == 0 || abs(lineRotation) == Long2Fix(90) || abs(lineRotation) == Long2Fix(180) ) {
		status = ATSUMeasureTextImage( iTextLayout, iLineOffset, iLineLength, iLocationX, iLocationY, &imageRect );
		if ( status == noErr ) {
			p1->x = Long2Fix(imageRect.left);	p1->y = Long2Fix(imageRect.top);
			p2->x = Long2Fix(imageRect.right);	p2->y = Long2Fix(imageRect.top);
			p3->x = Long2Fix(imageRect.right);	p3->y = Long2Fix(imageRect.bottom);
			p4->x = Long2Fix(imageRect.left);	p4->y = Long2Fix(imageRect.bottom);
		}
	} else {
		// Set the rotation to zero temporarily.
		status = atsuSetLineRotation( iTextLayout, 0 );
		require( status == noErr, EXIT );
		
		// Get the bounding box.
		status = ATSUMeasureTextImage( iTextLayout, iLineOffset, iLineLength, 0, 0, &imageRect );
		require( status == noErr, EXIT );
		
		if ( status == noErr ) {
			// rotate and translate it.
			Fixed lineAngle = FixMul(lineRotation, kATSRadiansFactor);
			Fixed rotateCos = Frac2Fix(FracCos(lineAngle));
			Fixed rotateSin = Frac2Fix(FracSin(lineAngle));
			
			RotateAndTranslate( Long2Fix(imageRect.left), Long2Fix(imageRect.top), rotateCos, rotateSin, iLocationX, iLocationY, p1 );
			RotateAndTranslate( Long2Fix(imageRect.right), Long2Fix(imageRect.top), rotateCos, rotateSin, iLocationX, iLocationY, p2 );
			RotateAndTranslate( Long2Fix(imageRect.right), Long2Fix(imageRect.bottom), rotateCos, rotateSin, iLocationX, iLocationY, p3 );
			RotateAndTranslate( Long2Fix(imageRect.left), Long2Fix(imageRect.bottom), rotateCos, rotateSin, iLocationX, iLocationY, p4 );
		}
		
		// Put the old rotation back
		if ( lineRotation != 0 ) {
			status = atsuSetLineRotation( iTextLayout, lineRotation );
		}
	}
	
EXIT:
	return status;
}

#else

OSStatus
atsuMeasureRotatedTextImage( ATSUTextLayout iTextLayout, 
							UniCharArrayOffset iLineOffset,
							UniCharCount iLineLength,
							ATSUTextMeasurement iLocationX,
							ATSUTextMeasurement iLocationY,
							atsuFixedPt *p1,
							atsuFixedPt *p2,
							atsuFixedPt *p3,
							atsuFixedPt *p4 )
{
/* 
   This version leaves the original layout alone, cloning the line in question into a new temporary layout to
   do the unrotated measurement.
   
   For small layouts, this can be slower than the above version.
 */
 
	OSStatus 		status = noErr;
	Fixed 			lineRotation = 0;	
	Rect 			imageRect;
	ATSUTextLayout	tempLayout = NULL;
		
	// Get the current rotation.
	status = atsuGetLineRotation( iTextLayout, &lineRotation );
	require( status == noErr, EXIT );
	
	// If the line rotation is a multiple of 90 degrees, the standard routine gives us the right answer.
	if ( lineRotation == 0 || abs(lineRotation) == Long2Fix(90) || abs(lineRotation) == Long2Fix(180) ) {
		status = ATSUMeasureTextImage( iTextLayout, iLineOffset, iLineLength, iLocationX, iLocationY, &imageRect );
		if ( status == noErr ) {
			p1->x = Long2Fix(imageRect.left);	p1->y = Long2Fix(imageRect.top);
			p2->x = Long2Fix(imageRect.right);	p2->y = Long2Fix(imageRect.top);
			p3->x = Long2Fix(imageRect.right);	p3->y = Long2Fix(imageRect.bottom);
			p4->x = Long2Fix(imageRect.left);	p4->y = Long2Fix(imageRect.bottom);
		}
	} else {
		// Create a new layout that covers the given lines
		
		status = atsuCloneTextLayout( iTextLayout, iLineOffset, iLineLength, true, &tempLayout );
		require( status == noErr, EXIT );
		
		// Force rotation to 0.
		status = atsuSetLineRotation( tempLayout, 0 );
		require( status == noErr, EXIT );
		
		// Get the bounding box
		status = ATSUMeasureTextImage( tempLayout, iLineOffset, iLineLength, 0, 0, &imageRect );
		
		if ( status == noErr ) {
			// rotate and translate it.
			Fixed lineAngle = FixMul(lineRotation, kATSRadiansFactor);
			Fixed rotateCos = Frac2Fix(FracCos(lineAngle));
			Fixed rotateSin = Frac2Fix(FracSin(lineAngle));
			
			RotateAndTranslate( Long2Fix(imageRect.left), Long2Fix(imageRect.top), rotateCos, rotateSin, iLocationX, iLocationY, p1 );
			RotateAndTranslate( Long2Fix(imageRect.right), Long2Fix(imageRect.top), rotateCos, rotateSin, iLocationX, iLocationY, p2 );
			RotateAndTranslate( Long2Fix(imageRect.right), Long2Fix(imageRect.bottom), rotateCos, rotateSin, iLocationX, iLocationY, p3 );
			RotateAndTranslate( Long2Fix(imageRect.left), Long2Fix(imageRect.bottom), rotateCos, rotateSin, iLocationX, iLocationY, p4 );
		}
	}
	
EXIT:
	if ( tempLayout ) (void) ATSUDisposeTextLayout( tempLayout );

	return status;
}

#endif



OSStatus atsuCloneTextLayout(
			ATSUTextLayout iTextLayout, UniCharArrayOffset iOffset, UniCharCount iLength, Boolean expandToLineBoundary,
			ATSUTextLayout *oTextLayout )
{
	OSStatus			status = noErr;
	void				*theText;
	Boolean				textIsHandle;
	UniCharArrayOffset	textOffset;
	UniCharCount		textLength;
	UniCharCount		textTotalLength;
	ATSUTextLayout		newLayout = NULL;
	UniCharArrayOffset	currentOffset, endOffset, breakOffset;
	ItemCount			breakCount;

	// First get our hands on the text info.
	status = ATSUGetTextLocation( iTextLayout, &theText, &textIsHandle, &textOffset, &textLength, &textTotalLength );
	require( status == noErr, EXIT );
	
	// Fix up the input range
	if ( iOffset == kATSUFromTextBeginning ) iOffset = textOffset;
	if ( iLength == kATSUToTextEnd ) iLength = textLength;
	
	if ( iOffset < textOffset ) {
		iLength -= (textOffset - iOffset);
		iOffset = textOffset;
	}
	if ( iLength > textLength )
		iLength = textLength;
	
	endOffset = iOffset + iLength;
	
	// Now adjust outwards to a line boundary if necessary.
	if ( expandToLineBoundary ) {
		// The easy one: find the first break after our range.
		status = ATSUGetSoftLineBreaks( iTextLayout, endOffset - 1, kATSUToTextEnd, 1, &breakOffset, &breakCount );
		require( status == noErr, EXIT );
		
		if ( breakCount > 0 )
			endOffset = breakOffset;
		else
			endOffset = textOffset + textLength;
		
		iLength = endOffset - iOffset;
				
		// Search for the first line break before this range.
		// We add one to the length so that a break just before the range gets picked up.
		breakOffset = textOffset;
		do {
			status = ATSUGetSoftLineBreaks( iTextLayout, breakOffset, iOffset - breakOffset + 1, 1, &breakOffset, &breakCount );
			require( status == noErr, EXIT );
		} while ( breakCount > 0 );
		
		iLength += iOffset - breakOffset;
		iOffset = breakOffset;
		
		endOffset = iOffset + iLength;
	}
	
	// Create the new layout.
	if ( textIsHandle )
		status = ATSUCreateTextLayoutWithTextHandle(
					(UniCharArrayHandle) theText, iOffset, iLength, textTotalLength, 0, NULL, NULL, &newLayout );
	else
		status = ATSUCreateTextLayoutWithTextPtr(
					(ConstUniCharArrayPtr) theText, iOffset, iLength, textTotalLength, 0, NULL, NULL, &newLayout );
	
	// Set the styles
	if ( status == noErr ) {
		currentOffset = iOffset;
		while ( (currentOffset < endOffset) && (status == noErr) ) {
			ATSUStyle			runStyle;
			UniCharArrayOffset	runStart;
			UniCharCount		runLength;
			
			// @@@ Will this work for an incomplete layout?
			status = ATSUGetRunStyle( iTextLayout, currentOffset, &runStyle, &runStart, &runLength );
			if ( status == noErr ) {
				// Adjust run bounds
				if ( runStart + runLength > endOffset ) {
					runLength -= runStart + runLength - endOffset;
				}
				if ( runStart < iOffset ) {
					runLength -= iOffset - runStart;
					runStart = iOffset;
				}
				status = ATSUSetRunStyle( newLayout, runStyle, runStart, runLength );
				currentOffset = runStart + runLength;	// NOT currentOffset + runLength !!!
			}
		}
	}
	
	// Copy the soft breaks
	if ( status == noErr ) {
		currentOffset = iOffset;
		while ( (currentOffset < endOffset) && (status == noErr) ) {
			
			status = ATSUGetSoftLineBreaks( iTextLayout, currentOffset, endOffset - currentOffset, 1, &breakOffset, &breakCount );
			if ( status == noErr && breakCount > 0 ) {
				status = ATSUSetSoftLineBreak( newLayout, breakOffset );
				currentOffset = breakOffset + 1;
			} else {
				currentOffset = endOffset;
			}
		}
	}
	
	// Copy layout controls
	if ( status == noErr )
		status = ATSUCopyLayoutControls( iTextLayout, newLayout );
	
	// Copy the refCon
	if ( status == noErr ) {
		UInt32 refCon;
		status = ATSUGetTextLayoutRefCon( iTextLayout, &refCon );
		if ( status == noErr )
			status = ATSUSetTextLayoutRefCon( newLayout, refCon );
	}

EXIT:	
	// Clean up if there was an error.
	if ( status != noErr && newLayout != NULL ) {
		(void) ATSUDisposeTextLayout( newLayout );
		newLayout = NULL;
	}
	
	*oTextLayout = newLayout;
	return status;
}



OSStatus
atsuDrawText( ConstUniCharArrayPtr	iTextBuf,
			  UniCharArrayOffset	iFirstChar,
			  UniCharCount			iCharCount )
{
	ATSUStyle		theStyle = NULL;
	ATSUTextLayout	theLayout = NULL;
	OSStatus		status = noErr;
	
	status = ATSUCreateStyle( &theStyle );
	require( status == noErr, EXIT );
	
	status = atsuSetStyleFromGrafPtr( theStyle, NULL, false );
	require( status == noErr, EXIT );
	
	status = atsuSetNoCaretAngle( theStyle, true );	// this is for the pen loc calculation later.
	require( status == noErr, EXIT );
	
	status = ATSUCreateTextLayoutWithTextPtr(
					iTextBuf, iFirstChar, iCharCount, iFirstChar + iCharCount,
					1, &iCharCount, &theStyle, &theLayout );
	require( status == noErr, EXIT );
	
	// We like to show all the text if possible.
	status = ATSUSetTransientFontMatching( theLayout, true );
	
	// @@@ Any other layout controls that should be forced?
	
	if ( status == noErr ) {
		// Now for the fun part. We need to figure out where to leave the pen when we're finished.
		// We've suppressed caret angles for two reasons: first, this prevents ATSUI from mucking with the last caret
		// position, and second, it makes the location on the baseline easy to compute.
		Boolean	lineDirection;
		Point	penLoc;
		ATSUCaret	theCaret;
		
		GetPen( &penLoc );
		status = atsuGetLineDirection( theLayout, &lineDirection );
		require( status == noErr, EXIT );
		
		// We need the rightmost caret position. The typographic measure is in ideal coordinates, which won't do
		// and the image bounds gives us the last bit position, not where the last advance takes us. We could
		// extract this from the highlight, but that means poking around inside a region. 
		status = ATSUOffsetToPosition( theLayout, lineDirection ? iFirstChar : iFirstChar + iCharCount, false,
										&theCaret, NULL, NULL );
		require( status == noErr, EXIT );
		
		// Finally, we get to draw the text
		status = ATSUDrawText(
					theLayout, kATSUFromTextBeginning, kATSUToTextEnd,
					kATSUUseGrafPortPenLoc, kATSUUseGrafPortPenLoc );
		if ( status == noErr ) 
			MoveTo( penLoc.h + Fix2Long( theCaret.fX ), penLoc.v );
	}

EXIT:	
	if ( theLayout ) (void) ATSUDisposeTextLayout( theLayout );
	if ( theStyle ) (void) ATSUDisposeStyle( theStyle );
	
	return status;
}
