/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	MoreATSUnicodeFonts.cp	-	Source file
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
------------------------------------------------------------------------------*/

#include "MoreATSUnicodeInternal.h"

#include <stddef.h>
#include <MacMemory.h>
#include <Menus.h>
#include <StringCompare.h>

#include "MoreATSUnicode.h"


typedef struct FontNamesRecord {
	ATSUFontID	fontID;
	ScriptCode	familyScript, styleScript;
	ItemCount	familyLength, styleLength;
	ByteOffset	styleOffset;
	char		names[ 1 ];
} FontNamesRecord, *FontNamesPtr;


#if !USE_NEW_TOOLBOX_CALLS

typedef struct FontMenuItemRecord {
	short		menuID;
	short		itemNum;
	ATSUFontID	fontID;
} FontMenuItemRecord, **FontMenuItemsHandle;

#endif


// @@@ Put in proper name search.
static FontNamesPtr
BuildFontNamesRecord( ATSUFontID fontID )
{
	OSStatus status = noErr;
	ByteCount		familyNameLength = 0, styleNameLength = 0;
	ItemCount		familyIndex, styleIndex;
	FontNamesPtr	fontNames = NULL;
	
	// Find the font's family name.
	status = ATSUFindFontName( fontID, kFontFamilyName, kFontMacintoshPlatform, kFontRomanScript, kFontNoLanguage,
							   0, NULL, &familyNameLength, &familyIndex );
	// Find the font's style name.
	if ( status == noErr )
		status = ATSUFindFontName( fontID, kFontStyleName, kFontMacintoshPlatform, kFontRomanScript, kFontNoLanguage,
								   0, NULL, &styleNameLength, &styleIndex );
	
	if ( status == noErr ) {
		fontNames = (FontNamesPtr) NewPtrClear( offsetof(FontNamesRecord, names) + familyNameLength + styleNameLength );
		if ( fontNames != NULL ) {
			fontNames->fontID = fontID;
			fontNames->familyScript = smRoman;
			(void) ATSUGetIndFontName( fontID, familyIndex, familyNameLength,
										 &fontNames->names[ 0 ], &fontNames->familyLength,
										 NULL, NULL, NULL, NULL );
			fontNames->styleOffset = familyNameLength;
			fontNames->styleScript = smRoman;
			(void) ATSUGetIndFontName( fontID, styleIndex, styleNameLength,
										 &fontNames->names[ familyNameLength ], &fontNames->styleLength,
										 NULL, NULL, NULL, NULL );
		}
	}
	
	// Filter out invisible fonts.
	if ( fontNames->names[ 0 ] == '.' || fontNames->names[ 0 ] == '%' )
		status = 1;
	
	if ( status != noErr && fontNames != NULL ) {
		DisposePtr( (Ptr) fontNames );
		fontNames = NULL;
	}
	
	return fontNames;
}



// The font list is sorted first by family name and then by style.
static int
CompareFontNames( FontNamesPtr names1, FontNamesPtr names2 )
{
	int order;
	
	order = TextOrder(
				&names1->names[ 0 ], &names2->names[ 0 ],
				names1->familyLength, names2->familyLength,
				names1->familyScript, names2->familyScript,
				currentDefLang, currentDefLang );
	if ( order == 0 )
		order = TextOrder(
					&names1->names[ names1->styleOffset ], &names2->names[ names2->styleOffset ],
					names1->styleLength, names2->styleLength,
					names1->styleScript, names2->styleScript,
					currentDefLang, currentDefLang );

	return order;
}



static void
InsertFontInSortedList( ATSUFontID fontID, FontNamesPtr sortedFontList[], ItemCount *listLength )
{
	OSStatus		status = noErr;
	FontNamesPtr	fontNames = NULL;
	int				high, low, insertIndex, order;
	
	fontNames = BuildFontNamesRecord( fontID );
	
	if ( fontNames ) {
		// Find the insert position.
		low = 0; high = *listLength; insertIndex = low + (high - low) / 2;
		while ( low < high ) {
			order = CompareFontNames( fontNames, sortedFontList[ insertIndex ] );
			if ( order < 0 ) {	// new font goes before probe value
				high = insertIndex;
			} else if ( order > 0 ) { // new font goes after probe value
				low = insertIndex + 1;
			} else { // same as probe value: don't enter duplicates in list
				insertIndex = -1;
				break;
			}
			insertIndex = low + (high - low) / 2;
		}
		
		if ( insertIndex >= 0 ) {
			// Add a new record.
			BlockMoveData( &sortedFontList[ insertIndex ], &sortedFontList[ insertIndex + 1 ], (*listLength - insertIndex) * sizeof(FontNamesPtr) );
			sortedFontList[ insertIndex ] = fontNames;
			*listLength += 1;
		}
	}

	if ( status != noErr && fontNames ) DisposePtr( (Ptr) fontNames );
}


static MenuHandle
NewMenuName( short menuID, long length, char title[] )
{
    Str255 buf;

    buf[ 0 ] = length < 255 ? length : 255;
    BlockMove( title, &buf[ 1 ], buf[ 0 ]);
    return NewMenu( menuID, buf );
}


static SInt16
AppendMenuName( MenuHandle menu, long length, char name[], ScriptCode script )
{
	Str255	buf;
	SInt16	itemNum;
	
	buf[ 0 ] = length > 255 ? 255 : length;
	BlockMoveData( name, &buf[ 1 ], buf[ 0 ] );	// @@@ truncation not quite right for two-byte scripts.

	AppendMenu( menu, "\psome text" );	// avoid metacharacter processing
	itemNum = CountMenuItems( menu );
	SetMenuItemText( menu, itemNum, buf );
#if USE_NEW_TOOLBOX_CALLS
	(void) SetMenuItemTextEncoding( menu, itemNum, script );
#else
	SetItemCmd( menuHandle, menuIndex, (char) 0x1c );			// show item has a script
	SetItemIcon( menuHandle, menuIndex, theID );				// scripts go in icon field
#endif
	
	return itemNum;
}


// Take the sorted font ID list and pour it into a hierarchial menu (appearance-savvy!)
// If we're using Appearance 1.1 or later, we simply store the fontID as a menu item refCon and don't bother building
// the adjunct font list.
static void
FontListToHierarchialMenu( MenuHandle theMenu, short firstHierMenuID, const FontNamesPtr fontList[], ItemCount numFonts, short *oNumSubmenus, void **lookupCookie )
{
	int 					i = 0, j, k, thisItem;
	short					submenusCreated = 0;
	
#if !USE_NEW_TOOLBOX_CALLS
	FontMenuItemsHandle		fontMenuItems = NULL;
	int						recordNum = 0;
	
	if ( numFonts > 0 ) {
		fontMenuItems = (FontMenuItemsHandle) NewHandle( numFonts * sizeof(FontMenuItemRecord) );
		if ( MemError() ) return;
	}
#endif
	
	while ( i < numFonts ) {
		// Collect up the fonts with the same family name
		for ( j = i + 1; j < numFonts; j++ ) {
			int order = TextOrder(
				&fontList[ i ]->names[ 0 ], &fontList[ j ]->names[ 0 ],
				fontList[ i ]->familyLength, fontList[ j ]->familyLength,
				fontList[ i ]->familyScript, fontList[ j ]->familyScript,
				currentDefLang, currentDefLang );

			if ( order != 0 )
				break;
		}
		
		// Should the style name be added to the top level if there's only one style? How about using full name instead?
		thisItem = AppendMenuName(
					theMenu, fontList[ i ]->familyLength, &fontList[ i ]->names[ 0 ], fontList[ i ]->familyScript );

		// Add a submenu for the styles
		if ( j - i > 1 ) {
			short subID = firstHierMenuID + submenusCreated++;
			MenuHandle subMenu = NewMenuName( subID, fontList[ i ]->familyLength, &fontList[ i ]->names[ 0 ] );
			if ( subMenu != NULL ) {
				for ( k = i; k < j; k++ ) {
					int subItemNum = AppendMenuName(
							subMenu, fontList[ k ]->styleLength, &fontList[ k ]->names[ fontList[ k ]->styleOffset ], fontList[ k ]->styleScript );
					// leave a trail so we can recover the fontID.
#if USE_NEW_TOOLBOX_CALLS
#pragma unused(lookupCookie)
					(void) SetMenuItemRefCon( subMenu, subItemNum, (UInt32) fontList[ k ]->fontID );
#else					
					(*fontMenuItems)[ recordNum ].fontID = fontList[ k ]->fontID;
					(*fontMenuItems)[ recordNum ].menuID = subID;
					(*fontMenuItems)[ recordNum ].itemNum = subItemNum;
					recordNum += 1;
#endif
				}
				InsertMenu( subMenu, hierMenu );
#if USE_NEW_TOOLBOX_CALLS
				(void) SetMenuItemHierarchicalID( theMenu, thisItem, subID );
#else
				SetItemCmd( theMenu, thisItem, hMenuCmd );
				SetItemMark( theMenu, thisItem, subID );
#endif
			}
		} else {
			// leave a trail so we can recover the fontID.
#if USE_NEW_TOOLBOX_CALLS
			(void) SetMenuItemRefCon( theMenu, thisItem, (UInt32) fontList[ i ]->fontID );
#else					
			(*fontMenuItems)[ recordNum ].fontID = fontList[ i ]->fontID;
			(*fontMenuItems)[ recordNum ].menuID = (**theMenu).menuID;
			(*fontMenuItems)[ recordNum ].itemNum = thisItem;
			recordNum += 1;
#endif
		}

		i = j;	// next family name
	}
	
	if ( oNumSubmenus ) *oNumSubmenus = submenusCreated;
#if !USE_NEW_TOOLBOX_CALLS
	*lookupCookie = fontMenuItems;
#endif
}


// Build a standard font menu.
OSStatus
atsuAppendFontMenu( MenuHandle theMenu, short firstHierMenuID, short *oSubmenuCount, void **lookupCookie )
{
	OSStatus		status = noErr;
	ItemCount		numFonts = 0;
	ATSUFontID		*fontIDList = NULL;
	FontNamesPtr	*sortedFontList = NULL;
	ItemCount		sortedListLength = 0;
	int				i;
	
	status = ATSUFontCount( &numFonts );
	if ( status == noErr && numFonts > 0 ) {
		fontIDList = (ATSUFontID*) NewPtrClear( numFonts * sizeof(ATSUFontID) );
		status = MemError();
		
		if ( status == noErr ) {
			sortedFontList = (FontNamesPtr*) NewPtrClear( numFonts * sizeof(FontNamesPtr) );
			status = MemError();
		}
		
		if ( status == noErr )
			status = ATSUGetFontIDs( fontIDList, numFonts, NULL );
		
		if ( status == noErr ) {
			for ( i = 0; i < numFonts; i++ ) {
				InsertFontInSortedList( fontIDList[ i ], sortedFontList, &sortedListLength );
			}
			
			FontListToHierarchialMenu( theMenu, firstHierMenuID, sortedFontList, sortedListLength, oSubmenuCount, lookupCookie );
		}
	}
	
	if ( fontIDList ) DisposePtr( (Ptr) fontIDList );
	if ( sortedFontList ) {
		for ( i = 0; i < sortedListLength; i++ )
			DisposePtr( (Ptr) sortedFontList[ i ] );
		DisposePtr( (Ptr) sortedFontList );
	}
	
	return status;
}


ATSUFontID
atsuGetFontForFontMenuItem( short menuID, short itemNum, void *menuLookupCookie )
{
#if USE_NEW_TOOLBOX_CALLS
#pragma unused(menuLookupCookie)
	// The font ID is stored as the menu item's refCon. Isn't that handy?
	MenuHandle	theMenu = GetMenuHandle( menuID );
	
	if ( theMenu != NULL ) {
		UInt32	refCon;
		OSStatus status = GetMenuItemRefCon( theMenu, itemNum, &refCon );
		if ( status == noErr ) return (ATSUFontID) refCon;
	}
	
	return kATSUInvalidFontID;
	
# else // !USE_NEW_TOOLBOX_CALLS
	// Have to search for it in the side data structure we built.
	
	FontMenuItemsHandle		fontMenuItems = (FontMenuItemsHandle) menuLookupCookie;
	short					numEntries = InlineGetHandleSize( (Handle) fontMenuItems ) / sizeof(FontMenuItemRecord);

// @@@ If we sorted the item list we could do a binary search.
#if 0
	short					low = 0, high = numEntries, probe = low + (high - low) / 2;
	
	while ( low < high ) {
		if ( menuID < (*fontMenuItems)[ probe ].menuID )
			high = probe;
		else if ( menuID > (*fontMenuItems)[ probe ].menuID )
			low = probe + 1;
		else if ( itemNum < (*fontMenuItems)[ probe ].itemNum )
			high = probe;
		else if ( itemNum > (*fontMenuItems)[ probe ].itemNum )
			low = probe + 1;
		else
			// found it!
			return (*fontMenuItems)[ probe ].fontID;

		probe = low + (high - low) / 2;
	}
	
#else

	// brute-force search.
	short					i;
	
	for ( i = 0; i < numEntries; ++i ) {
		if ( menuID == (*fontMenuItems)[ i ].menuID && itemNum == (*fontMenuItems)[ i ].itemNum )
			return (*fontMenuItems)[ i ].fontID;
	}
	
#endif

	// If we got here, we didn't find it.
	return kATSUInvalidFontID;

#endif	// if USE_NEW_TOOLBOX_CALLS
}


OSStatus
atsuDisposeFontMenuLookupCookie( void *menuLookupCookie )
{
	if ( menuLookupCookie ) {
		DisposeHandle( (Handle) menuLookupCookie );
		return MemError();
	} else {
		return noErr;
	}
}
