/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_2.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 2 of 'Demos' menu:
#	
#		Multiple styles in 1 single horizontal line
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_2.h"

OSErr NewSingleLineMultipleStylesWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 200, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pSingle Line & Multiple Styles", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeLongerUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// let's have 5 runs, setting the 4th word "much" and the last word "styles" in a different style
	wdataP->numberOfRuns = 5;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = 10;
	wdataP->runLengths[1] = 4;
	wdataP->runLengths[2] = 39;
	wdataP->runLengths[3] = 6;
	wdataP->runLengths[4] = wdataP->uTextLength - (10 + 4 + 39 + 6);

	// we only use 3 styles, the 3rd and 5th runs use the same one as the first
	// but we still have to dimension the styles array the same as the runLengths array
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// let's change the font
	TryToSetFontTo(tempS, "Helvetica");
	
	// let's add some attributes to the main style
	Fixed thePointSize = ff(12);
	ATSUAttributeTag theTag = kATSUSizeTag;
	ByteCount theSize = sizeof(Fixed);
	ATSUAttributeValuePtr theValuePtr = &thePointSize;
	status = ATSUSetAttributes(tempS, 1, &theTag, &theSize, &theValuePtr);
	if (status != noErr) DebugStr("\p ATSUSetAttributes failed");
	
	wdataP->styles[0] = tempS;
	
	ATSUStyle otempS;
	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[2] = otempS;

	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[4] = otempS;

	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
	
	// and let's add some different attributes to the 2nd style (vertical direction, italic, blue, size 48)
	thePointSize = ff(48);
	RGBColor blueColor = {0, 0, 0xFFFF};
	Boolean italic = true;
	ATSUVerticalCharacterType direction = kATSUStronglyVertical;
	ATSUAttributeTag tags[] = {kATSUSizeTag, kATSUColorTag, kATSUQDItalicTag, kATSUVerticalCharacterTag};
	ByteCount sizes[] = {sizeof(Fixed), sizeof(RGBColor), sizeof(Boolean), sizeof(ATSUVerticalCharacterType)};
	ATSUAttributeValuePtr values[] = {&thePointSize, &blueColor, &italic, &direction};
	status = ATSUSetAttributes(tempS, 4, tags, sizes, values);
	if (status != noErr) DebugStr("\p ATSUSetAttributes failed");
	
	wdataP->styles[1] = tempS;

	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// and let's add some different attributes to the 3rd style (Times 60 underline)
	thePointSize = ff(60);
	Boolean underline = true;
	ATSUFontID timesID = GetFontIDFromMacFontName("Times");
	ATSUAttributeTag tags2[] = {kATSUSizeTag, kATSUQDUnderlineTag, kATSUFontTag};
	ByteCount sizes2[] = {sizeof(Fixed), sizeof(Boolean), sizeof(ATSUFontID)};
	ATSUAttributeValuePtr values2[] = {&thePointSize, &underline, &timesID};
	if (timesID != kATSUInvalidFontID)
		status = ATSUSetAttributes(tempS, 3, tags2, sizes2, values2);
	else
		status = ATSUSetAttributes(tempS, 2, tags2, sizes2, values2);
	if (status != noErr) DebugStr("\p ATSUSetAttributes failed");

	wdataP->styles[3] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");	
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(aWind->portRect.bottom-30);
	
	return status;
	}
