/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_3.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 3 of 'Demos' menu:
#	
#		Multiple styles in 1 single vertical line
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_3.h"

#include "MoreATSUnicode.h"

//
// NOTE - NOTE - NOTE
//
// The source code for this function would be the same (but for the last 10 lines)
// as NewSingleLineMultipleStylesWindow except that it uses the More ATSUI routines
// to make it clearer. Please compare to the source code in SampleWindow_2.cp
//
OSErr NewSingleVerticalLineMultipleStylesWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 700, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pSingle Line & Multiple Styles", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeLongerUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// let's have 5 runs, setting the 4th word "much" and the last word "styles" in a different style
	wdataP->numberOfRuns = 5;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = 10;
	wdataP->runLengths[1] = 4;
	wdataP->runLengths[2] = 39;
	wdataP->runLengths[3] = 6;
	wdataP->runLengths[4] = wdataP->uTextLength - (10 + 4 + 39 + 6);

	// we only use 3 styles, the 3rd and 5th runs use the same one as the first
	// but we still have to dimension the styles array the same as the runLengths array
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// let's change the font
	TryToSetFontTo(tempS, "Helvetica");
	
	// let's add some attributes to the main style
	status = atsuSetSize(tempS, ff(12));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	
	wdataP->styles[0] = tempS;
	
	ATSUStyle otempS;
	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[2] = otempS;

	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[4] = otempS;

	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
	
	// and let's add some different attributes to the 2nd style (vertical direction, italic, blue, size 48)
	status = atsuSetSize(tempS, ff(48));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDItalic(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDItalic failed");
	status = atsuSetVerticalCharacter(tempS, kATSUStronglyVertical);
	if (status != noErr) DebugStr("\p atsuSetVerticalCharacter failed");
	RGBColor blueColor = {0, 0, 0xFFFF};
	status = atsuSetColor(tempS, blueColor);
	if (status != noErr) DebugStr("\p atsuSetColor failed");
		
	wdataP->styles[1] = tempS;

	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// and let's add some different attributes to the 3rd style (Times 60 underline)
	TryToSetFontTo(tempS, "Times");
	status = atsuSetSize(tempS, ff(60));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[3] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");	
	wdataP->textLayout = tempTL;
	
	// let's rotate the line
	status = atsuSetLineRotation(tempTL, ff(-90));
	if (status != noErr) DebugStr("\p atsuSetLineRotation failed");
	
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(30);
	wdataP->yLocation = ff(10);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 3;
	
	// let's add a button
	Rect buttonRect = {622, 540, 640, 610};
	ControlHandle button = NewControl(aWind, &buttonRect, "\pRotate!", true, 0, 0, 1, pushButProc, 1);
	
	return status;
	}

void RotateTheLine(WindowDataPtr wdataP)
	{
	GrafPtr currentPort;
	GetPort(&currentPort);
	EraseRect(&currentPort->portRect);
	
	// iterate angle and draw
	long i;
	for (i = 0; i >= -90; i -= 5)
		{
		OSStatus status = atsuSetLineRotation(wdataP->textLayout, ff(i));
		if (status != noErr) DebugStr("\p atsuSetLineRotation failed");

		status = ATSUDrawText(wdataP->textLayout, 0, wdataP->uTextLength, wdataP->xLocation, wdataP->yLocation);
		if (status != noErr) DebugStr("\p ATSUDrawText failed");
		}

	// ask for an update...
	EraseRect(&currentPort->portRect);
	InvalRect(&currentPort->portRect);
	}