/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_4.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 4 of 'Demos' menu:
#	
#		Multiple Paragraphs (all in a single style)
#		Break line management
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_4.h"

#include "MoreATSUnicode.h"

OSErr NewParagraphsWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 470, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pParagraphs", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some very long text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeVeryLongUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// only 1 style thus only 1 run
	wdataP->numberOfRuns = 1;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = wdataP->uTextLength;

	// and it's the default style
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Chicago");

	// let's set the font size
	status = atsuSetSize(tempS, ff(14));
	if (status != noErr) DebugStr("\p atsuSetSize failed");

	wdataP->styles[0] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(20);
	
	// now, let's break the text in pieces
	BreakTheTextInLines(aWind, wdataP);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 4;
	
	// let's add 2 buttons
	Rect button1Rect = {392, 540, 410, 610};
	ControlHandle button1 = NewControl(aWind, &button1Rect, "\pBigger", true, 0, 0, 1, pushButProc, 1);
	Rect button2Rect = {392, 460, 410, 530};
	ControlHandle button2 = NewControl(aWind, &button2Rect, "\pSmaller", true, 0, 0, 1, pushButProc, 2);
	
	return status;
	}

void BreakTheTextInLines(WindowPtr aWind, WindowDataPtr wdataP)
	{
	OSStatus status;
	ATSUTextMeasurement lineWidth = ff(aWind->portRect.right - aWind->portRect.left - 20);
	UniCharArrayOffset offset = 0, endLineOffset;
	ItemCount nbLines = 0;
	ATSUTextMeasurement ascent, descent;
	
	wdataP->maxAscent = wdataP->maxDescent = 0;
	
	// measure and break each line, thus setting a soft line break that we'll grab back later
	while (offset < wdataP->uTextLength)
		{
		status = ATSUBreakLine(wdataP->textLayout, offset, lineWidth, true, &endLineOffset);
		if (status != noErr) DebugStr("\p ATSUBreakLine failed");
		
		// it could be that the line width is so narrow that even a single character won't fit.
		// in that case, ATSUBreakLine returns noErr but endLineOffset returns equal to offset.
		// if that's the case, let's break out...
		if (endLineOffset == offset) break;
		
		// let's get ascent & descent to calculate the line height later.
		status = ATSUMeasureText(wdataP->textLayout, offset, endLineOffset-offset, NULL, NULL, &ascent, &descent);
		if (status != noErr) DebugStr("\p ATSUMeasureText failed");

		if (ascent > wdataP->maxAscent) wdataP->maxAscent = ascent;
		if (descent > wdataP->maxDescent) wdataP->maxDescent = descent;
		
		offset = endLineOffset;
		nbLines++;
		}
	
	// let's grab back all the soft line breaks in the array we'll keep around
	UniCharArrayOffset *endOfLines = (UniCharArrayOffset *)NewPtr(nbLines * sizeof(UniCharArrayOffset));
	if (endOfLines == NULL) DebugStr("\p NewPtr failed");
	ItemCount softBreakCount;
	status = ATSUGetSoftLineBreaks(wdataP->textLayout, 0, wdataP->uTextLength, nbLines, endOfLines, &softBreakCount);

	// the number of softbreaks should always be one less than the number of lines
	// since ATSUBreakLine does not insert a softbreak at the end of the text.
	if ((status != noErr) || (softBreakCount != nbLines-1)) DebugStr("\p ATSUGetSoftLineBreaks failed");
	
	// so let's set the last entry of the array
	endOfLines[softBreakCount] = wdataP->uTextLength;

	wdataP->endOfLines = endOfLines;
	wdataP->numberOfLines = nbLines;
	wdataP->lineHeight = wdataP->maxAscent + wdataP->maxDescent + ff(2);
	}

void SmallerBiggerFont(WindowDataPtr wdataP, Fixed variation)
	{
	OSStatus status;
	GrafPtr currentPort;
	GetPort(&currentPort);

	// create a Style container
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// grab the continuous style of our text layout
	status = ATSUGetContinuousAttributes(wdataP->textLayout, 0, wdataP->uTextLength, tempS);
	if (status != noErr) DebugStr("\p ATSUGetContinuousAttributes failed");

	// get the current font size
	Fixed thePointSize;
	status = atsuGetSize(tempS, &thePointSize);
	if (status != noErr) DebugStr("\p atsuGetSize failed");

	// add the variation
	thePointSize += variation;
	
	// let's have reasonable values:
	if (variation > 0)
		if (thePointSize > ff(24)) return;
	if (variation < 0)
		if (thePointSize < ff(4)) return;

	// set the new font size in the style
	status = atsuSetSize(tempS, thePointSize);
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	
	// apply the style to the text layout
	status = ATSUSetRunStyle(wdataP->textLayout, tempS, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUSetRunStyle failed");
	
	// clear all the current soft breaks
	status = ATSUClearSoftLineBreaks(wdataP->textLayout, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUClearSoftLineBreaks failed");
	
	// dispose of the current endOfLines array
	if (wdataP->endOfLines != NULL) DisposePtr((Ptr)wdataP->endOfLines);
	
	// and let's break the text in pieces again
	BreakTheTextInLines(currentPort, wdataP);

	// ask for an update...
	EraseRect(&currentPort->portRect);
	InvalRect(&currentPort->portRect);
	}

//
// Another way to modify the sizes:
//
/*
void SmallerBiggerFont(WindowDataPtr wdataP, Fixed variation)
	{
	OSStatus status;
	GrafPtr currentPort;
	GetPort(&currentPort);

	// create a Style container
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// grab the continuous style of our text layout
	status = ATSUGetContinuousAttributes(wdataP->textLayout, 0, wdataP->uTextLength, tempS);
	if (status != noErr) DebugStr("\p ATSUGetContinuousAttributes failed");

	// get the current font size
	Fixed thePointSize;
	ByteCount actSize, theSize = sizeof(Fixed);
	ATSUAttributeValuePtr theValuePtr = &thePointSize;
	status = ATSUGetAttribute(tempS, kATSUSizeTag, theSize, theValuePtr, &actSize);
	if (status != noErr) DebugStr("\p ATSUGetAttribute failed");

	// add the variation
	thePointSize += variation;
	
	// let's have reasonable values:
	if (variation > 0)
		if (thePointSize > ff(24)) return;
	if (variation < 0)
		if (thePointSize < ff(4)) return;

	// set the new font size in the style
	ATSUAttributeTag theTag = kATSUSizeTag;
	status = ATSUSetAttributes(tempS, 1, &theTag, &theSize, &theValuePtr);
	if (status != noErr) DebugStr("\p ATSUSetAttributes failed");
	
	// apply the style to the text layout
	status = ATSUSetRunStyle(wdataP->textLayout, tempS, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUSetRunStyle failed");
	
	// clear all the current soft breaks
	status = ATSUClearSoftLineBreaks(wdataP->textLayout, 0, wdataP->uTextLength);
	
	// dispose of the current endOfLines array
	if (wdataP->endOfLines != NULL) DisposePtr((Ptr)wdataP->endOfLines);
	
	// and let's break the text in pieces again
	BreakTheTextInLines(currentPort, wdataP);

	// ask for an update...
	EraseRect(&currentPort->portRect);
	InvalRect(&currentPort->portRect);
	}
*/