/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_5.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 5 of 'Demos' menu:
#	
#		Hilighting and Caret/Cursor movements
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_4.h"
#include "SampleWindow_5.h"

#include "MoreATSUnicode.h"

OSErr NewHighlightingCaretsCursorMovementsWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 470, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pHilighting, Carets & Cursor movements", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some very long text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeMixedUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// only 3 styles thus only 5 runs
	wdataP->numberOfRuns = 5;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = 243;
	wdataP->runLengths[1] = 36;
	wdataP->runLengths[2] = 143;
	wdataP->runLengths[3] = 43;
	wdataP->runLengths[4] = wdataP->uTextLength - 243 - 36 - 143 - 43;

	// and it's the default style
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Times");

	// let's set the font size
	status = atsuSetSize(tempS, ff(16));
	if (status != noErr) DebugStr("\p atsuSetSize failed");

	wdataP->styles[0] = tempS;
	
	ATSUStyle otempS;
	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[2] = otempS;

	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[4] = otempS;

	// second style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Helvetica");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[1] = tempS;

	// third style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Courier");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDItalic(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDItalic failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[3] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(20);
	
	// now, let's break the text in pieces
	BreakTheTextInLines(aWind, wdataP);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 5;
	
	// let's add 6 buttons
	Rect button1Rect = {392, 490, 410, 610};
	ControlHandle button1 = NewControl(aWind, &button1Rect, "\pHilight next 30", true, 0, 0, 1, pushButProc, 1);
	Rect button2Rect = {392, 340, 410, 480};
	ControlHandle button2 = NewControl(aWind, &button2Rect, "\pMove to next word", true, 0, 0, 1, pushButProc, 2);
	Rect button3Rect = {392, 160, 410, 330};
	ControlHandle button3 = NewControl(aWind, &button3Rect, "\pMove to previous word", true, 0, 0, 1, pushButProc, 3);
	Rect button4Rect = {392, 90, 410, 150};
	ControlHandle button4 = NewControl(aWind, &button4Rect, "\pHome", true, 0, 0, 1, pushButProc, 4);

	Rect button5Rect = {392, 50, 410, 80};
	ControlHandle button5 = NewControl(aWind, &button5Rect, "\p++", true, 0, 0, 1, pushButProc, 5);
	Rect button6Rect = {392, 10, 410, 40};
	ControlHandle button6 = NewControl(aWind, &button6Rect, "\p--", true, 0, 0, 1, pushButProc, 6);
	
	// some default values for our private data...
	wdataP->privateData = (void *)NewPtr(sizeof(Sample5_PrivateData));
	((Sample5_PrivateData *)wdataP->privateData)->withHilight = false;
	((Sample5_PrivateData *)wdataP->privateData)->caret = 0;
	((Sample5_PrivateData *)wdataP->privateData)->length = 30;
	((Sample5_PrivateData *)wdataP->privateData)->numberOfSelectedLines = 0;
	((Sample5_PrivateData *)wdataP->privateData)->selBlocks = NULL;
	
	return status;
	}

void DisposeSample5Window(WindowDataPtr wdataP)
	{
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	if (priv->selBlocks != NULL) DisposePtr((Ptr)priv->selBlocks);
	}

void SetSelectedBlocks(WindowDataPtr wdataP)
	{
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	ItemCount i, firstLine = -1, lastLine = -1;
	
	// find the first line where the selection starts
	for (i = 0; (i < wdataP->numberOfLines) && (firstLine == -1); i++)
		if (priv->caret < wdataP->endOfLines[i]) firstLine = i;
		
	// find the last line where the selection ends
	for (i = firstLine; (i < wdataP->numberOfLines) && (lastLine == -1); i++)
		if ((priv->caret+priv->length) < wdataP->endOfLines[i]) lastLine = i;
	
	// let's allocate (deallocating first if it's previously been allocated)...
	priv->numberOfSelectedLines = lastLine - firstLine + 1;
	if (priv->selBlocks != NULL) DisposePtr((Ptr)priv->selBlocks);
	priv->selBlocks = (SelectedBlock *)NewPtr(priv->numberOfSelectedLines * sizeof(SelectedBlock));
	
	// and fill the values
	UniCharArrayOffset pos = priv->caret;
	UniCharCount length, lengthToSelect = priv->length;
	for (i = 0; i < priv->numberOfSelectedLines; i++)
		{
		SelectedBlock *sb = &(priv->selBlocks[i]);
		sb->selStart = pos;
		length = wdataP->endOfLines[firstLine + i] - pos;
		if (length >= lengthToSelect) length = lengthToSelect;
		sb->selLength = length;
		lengthToSelect -= length;
		pos = wdataP->endOfLines[firstLine + i];
		sb->xLineSel = wdataP->xLocation;
		sb->yLineSel = wdataP->yLocation + (firstLine + i) * wdataP->lineHeight;
		}
	}

void HilightAndOrMove(WindowDataPtr wdataP, long what)
	{
	OSStatus status = noErr;
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	switch (what)
		{
		case 1:
			// first let's unhilight the current hilight if there's one
			UnhilightCurrentSelection(wdataP);

			// let's find out on how many lines the selection extends
			if ((wdataP->numberOfLines == 1) || (wdataP->endOfLines == NULL))
				{
				// this one is simple...
				priv->numberOfSelectedLines = 1;
				priv->selBlocks = (SelectedBlock *)NewPtr(priv->numberOfSelectedLines * sizeof(SelectedBlock));
				priv->selBlocks[0].selLength = priv->length;
				priv->selBlocks[0].selStart = priv->caret;
				priv->selBlocks[0].xLineSel = wdataP->xLocation;
				priv->selBlocks[0].yLineSel = wdataP->yLocation;
				}
			else
				{
				ItemCount i, firstLine = -1, lastLine = -1;
				
				// find the first line where the selection starts
				for (i = 0; (i < wdataP->numberOfLines) && (firstLine == -1); i++)
					if (priv->caret < wdataP->endOfLines[i]) firstLine = i;
					
				// find the last line where the selection ends
				for (i = firstLine; (i < wdataP->numberOfLines) && (lastLine == -1); i++)
					if ((priv->caret+priv->length) < wdataP->endOfLines[i]) lastLine = i;
				
				// let's allocate (deallocating first if it's previously been allocated)...
				priv->numberOfSelectedLines = lastLine - firstLine + 1;
				if (priv->selBlocks != NULL) DisposePtr((Ptr)priv->selBlocks);
				priv->selBlocks = (SelectedBlock *)NewPtr(priv->numberOfSelectedLines * sizeof(SelectedBlock));
				
				// and fill the values
				UniCharArrayOffset pos = priv->caret;
				UniCharCount length, lengthToSelect = priv->length;
				for (i = 0; i < priv->numberOfSelectedLines; i++)
					{
					SelectedBlock *sb = &(priv->selBlocks[i]);
					sb->selStart = pos;
					length = wdataP->endOfLines[firstLine + i] - pos;
					if (length >= lengthToSelect) length = lengthToSelect;
					sb->selLength = length;
					lengthToSelect -= length;
					pos = wdataP->endOfLines[firstLine + i];
					sb->xLineSel = wdataP->xLocation;
					sb->yLineSel = wdataP->yLocation + (firstLine + i) * wdataP->lineHeight;
					}
				}

			// let's hilight the selection we just computed
			HilightCurrentSelection(wdataP);
			priv->withHilight = true;

			// move the caret by priv->length for next time or go back to home if end reached.
			priv->caret += priv->length;
			if ((priv->caret+priv->length) >= wdataP->uTextLength) priv->caret = 0;
			
			break;
		case 2:
			{
			// first let's unhilight the current hilight if there's one
			UnhilightCurrentSelection(wdataP);

			// first let's move the caret back priv->length chars if needed and then search for the first word forward
			if (priv->withHilight) priv->caret -= priv->length;
			if (priv->caret < 0) priv->caret = 0;
			UniCharArrayOffset pos;
			status = ATSUNextCursorPosition(wdataP->textLayout, priv->caret, kATSUByWord, &pos);
			if (status != noErr)
				{
				DebugStr("\p ATSUNextCursorPosition failed");
				priv->caret = 0;
				}
			else priv->caret = ((pos+priv->length) >= wdataP->uTextLength)?0:pos;

			priv->withHilight = false;
			HilightAndOrMove(wdataP, 1);
			}
			break;
		case 3:
			{
			// first let's unhilight the current hilight if there's one
			UnhilightCurrentSelection(wdataP);

			// first let's move the caret back priv->length chars if needed and then search for the first word backward
			if (priv->withHilight) priv->caret -= priv->length;
			if (priv->caret < 0) priv->caret = 0;
			UniCharArrayOffset pos;
			status = ATSUPreviousCursorPosition(wdataP->textLayout, priv->caret, kATSUByWord, &pos);
			if (status != noErr)
				{
				DebugStr("\p ATSUNextCursorPosition failed");
				priv->caret = 0;
				}
			else priv->caret = pos;

			priv->withHilight = false;
			HilightAndOrMove(wdataP, 1);
			}
			break;
		case 4:
			// first let's unhilight the current hilight if there's one
			UnhilightCurrentSelection(wdataP);

			// and move the caret home.
			priv->caret = 0;

			priv->withHilight = false;
			break;
		}
	}

// To correctly hilight or unhilight the current selection when the window is
// activated, deactivated and/or updated, we have to complete the default
// behavior.

// Since a double Hilight means no hilight, we have to take extra care with the
// update region when activating/updating the window.

void ActivateSample5Window(WindowPtr theWind, WindowDataPtr wdataP, Boolean active)
	{
	// let's unhilight or rehilight the current hilight if there's one
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	if (priv->withHilight)
		{
		GrafPtr savePort;
		GetPort(&savePort);
		SetPort(theWind);
		
		// first save the current clip
		RgnHandle saveClipRgn = NewRgn();
		GetClip(saveClipRgn);
		
		// copy the update region to a new region and move it in local coordinate space
		Point thePoint = {0, 0};
		LocalToGlobal(&thePoint);
		RgnHandle updateRgn = NewRgn();
		CopyRgn(((WindowPeek)theWind)->updateRgn, updateRgn);
		OffsetRgn(updateRgn, -thePoint.h, -thePoint.v);
		
		// since the update mechanism will take care of hilighting within the update region
		// we want to take care here of the visible region minus the update region 
		RgnHandle clipRgn = NewRgn();
		DiffRgn(theWind->visRgn, updateRgn, clipRgn);
		SetClip(clipRgn);
		
		if (!active)
			UnhilightCurrentSelection(wdataP);
		else
			HilightCurrentSelection(wdataP);
		
		SetClip(saveClipRgn);
		DisposeRgn(clipRgn);
		SetPort(savePort);
		}
	}

void UpdateSample5Window(WindowPtr theWind, WindowDataPtr wdataP)
	{
	// let's hilight the current hilight if there's one (and if the window is active)
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	if ((priv->withHilight) && ((WindowPeek)theWind)->hilited)
		{
		// we only hilight in the visRgn (which was the update region before BeginUpdate)
		// because the activate mechanism took care of the rest.
		RgnHandle saveClipRgn = NewRgn();
		GetClip(saveClipRgn);
		SetClip(theWind->visRgn);
		HilightCurrentSelection(wdataP);
		SetClip(saveClipRgn);
		}
	}

void HilightCurrentSelection(WindowDataPtr wdataP)
	{
	OSStatus status = noErr;
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	ItemCount i;
	for (i = 0; i < priv->numberOfSelectedLines; i++)
		{
		SelectedBlock *sb = &(priv->selBlocks[i]);
		status = ATSUHighlightText(wdataP->textLayout, sb->xLineSel, sb->yLineSel, sb->selStart, sb->selLength);
		if (status != noErr) DebugStr("\p ATSUHighlightText failed");
		}
	}

void UnhilightCurrentSelection(WindowDataPtr wdataP)
	{
	Sample5_PrivateData *priv = (Sample5_PrivateData *)wdataP->privateData;
	if (!priv->withHilight) return;

	OSStatus status = noErr;
	ItemCount i;
	for (i = 0; i < priv->numberOfSelectedLines; i++)
		{
		SelectedBlock *sb = &(priv->selBlocks[i]);
		status = ATSUUnhighlightText(wdataP->textLayout, sb->xLineSel, sb->yLineSel, sb->selStart, sb->selLength);
		if (status != noErr) DebugStr("\p ATSUUnhighlightText failed");
		}
	}

void BetterSmallerBiggerFont(WindowDataPtr wdataP, Fixed variation)
	{
	OSStatus status;
	GrafPtr currentPort;
	GetPort(&currentPort);

	// create a Style container
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// iterate over the text layout to grab all the styles
	ItemCount i = 0;
	UniCharArrayOffset here = 0, start;
	UniCharCount length;
	status = ATSUGetRunStyle(wdataP->textLayout, here, &tempS, &start, &length);
	if (status != noErr)
		{
		DebugStr("\p ATSUGetRunStyle failed");
		return;
		}
	while (here < wdataP->uTextLength)
		{
		// get the current font size
		Fixed thePointSize;
		status = atsuGetSize(tempS, &thePointSize);
		if (status != noErr) DebugStr("\p atsuGetSize failed");

		// add the variation
		thePointSize += variation;
		
		// let's have reasonable values:
		if (variation > 0)
			if (thePointSize > ff(24)) thePointSize = ff(24);
		if (variation < 0)
			if (thePointSize < ff(4)) thePointSize = ff(4);

		// set the new font size in the style
		status = atsuSetSize(tempS, thePointSize);
		if (status != noErr) DebugStr("\p atsuSetSize failed");
		
		// apply the style to the text layout
		status = ATSUSetRunStyle(wdataP->textLayout, tempS, start, length);
		if (status != noErr) DebugStr("\p ATSUSetRunStyle failed");
		
		here = start + length;
		status = ATSUGetRunStyle(wdataP->textLayout, here, &tempS, &start, &length);
		if (status != noErr)
			{
			DebugStr("\p ATSUGetRunStyle failed");
			here = wdataP->uTextLength;
			}
		}
	
	// clear all the current soft breaks
	status = ATSUClearSoftLineBreaks(wdataP->textLayout, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUClearSoftLineBreaks failed");
	
	// dispose of the current endOfLines array
	if (wdataP->endOfLines != NULL) DisposePtr((Ptr)wdataP->endOfLines);
	
	// and let's break the text in pieces again
	BreakTheTextInLines(currentPort, wdataP);

	// ask for an update...
	EraseRect(&currentPort->portRect);
	InvalRect(&currentPort->portRect);
	}
