/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_6.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 6 of 'Demos' menu:
#	
#		Hit Testing
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_4.h"
#include "SampleWindow_5.h"
#include "SampleWindow_6.h"

#include "MoreATSUnicode.h"

OSErr NewHitTestingWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 470, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pHit Testing", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some very long text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeOtherMixedUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// only 3 styles thus only 5 runs
	wdataP->numberOfRuns = 5;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = 243;
	wdataP->runLengths[1] = 36;
	wdataP->runLengths[2] = 143;
	wdataP->runLengths[3] = 43;
	wdataP->runLengths[4] = wdataP->uTextLength - 243 - 36 - 143 - 43;

	// and it's the default style
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Times");

	// let's set the font size
	status = atsuSetSize(tempS, ff(16));
	if (status != noErr) DebugStr("\p atsuSetSize failed");

	wdataP->styles[0] = tempS;
	
	ATSUStyle otempS;
	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[2] = otempS;

	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[4] = otempS;

	// second style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Helvetica");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[1] = tempS;

	// third style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Courier");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDItalic(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDItalic failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[3] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(20);
	
	// now, let's break the text in pieces
	BreakTheTextInLines(aWind, wdataP);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 6;
	
	// some default values for our private data...
	wdataP->privateData = (void *)NewPtr(sizeof(Sample6_PrivateData));
	((Sample6_PrivateData *)wdataP->privateData)->withHilight = false;
	((Sample6_PrivateData *)wdataP->privateData)->caret = 0;
	((Sample6_PrivateData *)wdataP->privateData)->length = 0;
	((Sample6_PrivateData *)wdataP->privateData)->numberOfSelectedLines = 0;
	((Sample6_PrivateData *)wdataP->privateData)->selBlocks = NULL;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickTime = 0;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickPoint.h = -50;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickPoint.v = -50;
	
	return status;
	}

UniCharArrayOffset WhereInTextIsThisPoint(WindowDataPtr wdataP, Point thePoint, Boolean *leading)
	{
	OSStatus status = noErr;
	UniCharArrayOffset result;

	// let's test for the out of bounds first
	if (ff(thePoint.v) < (wdataP->yLocation - wdataP->maxAscent))
		result = 0;
	else if (ff(thePoint.v) > (wdataP->yLocation + ((wdataP->numberOfLines - 1) * wdataP->lineHeight) + wdataP->maxDescent))
		result = wdataP->uTextLength - 1;
	else
		{
		// determine first which line
		ItemCount theLine = (ff(thePoint.v) - (wdataP->yLocation - wdataP->maxAscent)) / wdataP->lineHeight;
		
		// which gives us the base line start
		ATSUTextMeasurement xl = wdataP->xLocation;
		ATSUTextMeasurement yl = wdataP->yLocation + theLine * wdataP->lineHeight;
		
		// and we call the right stuff
		UniCharArrayOffset there, notThere;
		// we seed the offset with the first Character Offset of that line
		if (theLine == 0)
			there = 0;
		else
			there = wdataP->endOfLines[theLine-1];
		xl = ff(thePoint.h) - xl;
		yl = ff(thePoint.v) - yl;
		status = ATSUPositionToOffset(wdataP->textLayout, xl, yl, &there, leading, &notThere);
		if (status != noErr)
			{
			DebugStr("\p ATSUPositionToOffset failed");
			result = 0;
			}
		else result = there;
		}
	
	// sanity check
	if (result < 0) result = 0;
	if (result > (wdataP->uTextLength - 1)) result = wdataP->uTextLength - 1;

	return result;
	}

void SelectWord(WindowDataPtr wdataP, UniCharArrayOffset thePos, Boolean leading, UniCharArrayOffset *startSel, UniCharArrayOffset *endSel)
	{
	OSStatus status = noErr;
	Sample6_PrivateData *priv = (Sample6_PrivateData *)wdataP->privateData;

	// let's get the end and the start of the word
	status = ATSUNextCursorPosition(wdataP->textLayout, thePos, kATSUByWord, endSel);
	if (status != noErr) DebugStr("\p ATSUNextCursorPosition failed");
	status = ATSUPreviousCursorPosition(wdataP->textLayout, thePos, kATSUByWord, startSel);
	if (status != noErr) DebugStr("\p ATSUPreviousCursorPosition failed");

	// check that we don't go beyond the edge of the word if we clicked on
	// the leading edge of the first character
	UniCharArrayOffset test, realStart = *startSel, realEnd = *endSel;
	status = ATSUPreviousCursorPosition(wdataP->textLayout, *endSel, kATSUByWord, &test);
	if (status != noErr) DebugStr("\p ATSUPreviousCursorPosition failed");
	if ((test == thePos) && leading && (*startSel < thePos)) realStart = thePos;

	// or the non-leading edge of the last character
	status = ATSUNextCursorPosition(wdataP->textLayout, *startSel, kATSUByWord, &test);
	if (status != noErr) DebugStr("\p ATSUNextCursorPosition failed");
	if ((test == thePos) && (!leading) && (*endSel > thePos)) realEnd = thePos;

	// always a sanity check
	if (realStart < 0) realStart = 0;
	if (realEnd > (wdataP->uTextLength - 1)) realEnd = wdataP->uTextLength - 1;
	
	// and we set all the proper values.
	priv->caret = realStart;
	priv->length = realEnd - realStart;
	*startSel = realStart;
	*endSel = realEnd;
	}

Boolean GotDoubleClick(WindowDataPtr wdataP, EventRecord *theEvent)
	{
	// no comment
	Sample6_PrivateData *priv = (Sample6_PrivateData *)wdataP->privateData;
	long deltaH = theEvent->where.h - priv->lastClickPoint.h;
	long deltaV = theEvent->where.v - priv->lastClickPoint.v;
	if (deltaH < 0) deltaH = -deltaH;
	if (deltaV < 0) deltaV = -deltaV;
	if (	(deltaH < 5) &&
			(deltaV < 5) &&
			((theEvent->when - priv->lastClickTime) < GetDblTime())		)
		return true;
	return false;
	}

void ClickInSample6Window(WindowDataPtr wdataP, EventRecord *theEvent)
	{
	OSStatus status = noErr;
	Sample6_PrivateData *priv = (Sample6_PrivateData *)wdataP->privateData;
	Boolean leading, withShift = ((theEvent->modifiers & shiftKey) != 0);
	Point thePoint = theEvent->where;
	GlobalToLocal(&thePoint);
	UniCharArrayOffset newPos, startSel, endSel, anchor;
	
	// unhilight current selection if any first
	UnhilightCurrentSelection(wdataP);
	newPos = WhereInTextIsThisPoint(wdataP, thePoint, &leading);
	if (!withShift)
		{
		// test for double-click
		if (GotDoubleClick(wdataP, theEvent))
			{
			SelectWord(wdataP, newPos, leading, &startSel, &endSel);
			anchor = newPos;
			}
		else
			{
			// if simple click, then disable current selection if any first
			priv->withHilight = false;
			priv->length = 0;
			priv->caret = newPos;
			anchor = startSel = endSel = priv->caret;
			
			priv->lastClickTime = theEvent->when;
			priv->lastClickPoint = theEvent->where;
			}
		}
	else
		{
		// if shift click, then extends the current selection if any
		startSel = priv->caret;
		endSel = startSel + priv->length;
		if (newPos < startSel)
			{
			startSel = newPos;
			anchor = endSel;
			}
		else
			{
			endSel = newPos;
			anchor = startSel;
			}
		priv->caret = startSel;
		priv->length = endSel - startSel;
		}

	if (priv->length > 0)
		{
		SetSelectedBlocks(wdataP);
		HilightCurrentSelection(wdataP);
		priv->withHilight = true;
		}
	
	// as long as the mouse button is down, dragging...
	Point lastPoint = thePoint;
	UniCharArrayOffset oldPos = newPos;
	while (StillDown())
		{
		GetMouse(&thePoint);
		if ((*((long *)&thePoint)) == (*((long *)&lastPoint))) continue;
		lastPoint = thePoint;
		newPos = WhereInTextIsThisPoint(wdataP, thePoint, &leading);
		if (newPos == oldPos) continue;
		oldPos = newPos;

		if (newPos < anchor)
			{
			startSel = newPos;
			endSel = anchor;
			}
		else
			{
			endSel = newPos;
			startSel = anchor;
			}
		priv->caret = startSel;
		priv->length = endSel - startSel;
		
		UnhilightCurrentSelection(wdataP);
		SetSelectedBlocks(wdataP);
		HilightCurrentSelection(wdataP);
		priv->withHilight = true;
		}
	}
