/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_7.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 7 of 'Demos' menu:
#	
#		Clipboard Management
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_4.h"
#include "SampleWindow_5.h"
#include "SampleWindow_6.h"
#include "SampleWindow_7.h"

#include "MoreATSUnicode.h"
#include "ATSUTextConversion.h"

WindowPtr gClipboardWindow = NULL;

Rect r1 = {  9, 8, 102, 612};
Rect r2 = {112, 8, 205, 612};
Rect r3 = {215, 8, 308, 612};
Rect r4 = {318, 8, 411, 612};

OSErr NewClipboardUsingWindow()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 470, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pClipboard Testing", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some very long text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeMixedAgainUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// only 3 styles thus only 5 runs
	wdataP->numberOfRuns = 5;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = 243;
	wdataP->runLengths[1] = 36;
	wdataP->runLengths[2] = 143;
	wdataP->runLengths[3] = 43;
	wdataP->runLengths[4] = wdataP->uTextLength - 243 - 36 - 143 - 43;

	// and it's the default style
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Times");

	// let's set the font size
	status = atsuSetSize(tempS, ff(16));
	if (status != noErr) DebugStr("\p atsuSetSize failed");

	wdataP->styles[0] = tempS;
	
	ATSUStyle otempS;
	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[2] = otempS;

	status = ATSUCreateAndCopyStyle(tempS, &otempS);
	if (status != noErr) DebugStr("\p ATSUCreateAndCopyStyle failed");
	wdataP->styles[4] = otempS;

	// second style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Helvetica");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[1] = tempS;

	// third style
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Courier");

	// let's set different style attributes
	status = atsuSetSize(tempS, ff(20));
	if (status != noErr) DebugStr("\p atsuSetSize failed");
	status = atsuSetQDItalic(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDItalic failed");
	status = atsuSetQDUnderline(tempS, true);
	if (status != noErr) DebugStr("\p atsuSetQDUnderline failed");

	wdataP->styles[3] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(20);
	
	// now, let's break the text in pieces
	BreakTheTextInLines(aWind, wdataP);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 7;
	
	// some default values for our private data...
	wdataP->privateData = (void *)NewPtr(sizeof(Sample6_PrivateData));
	((Sample6_PrivateData *)wdataP->privateData)->withHilight = false;
	((Sample6_PrivateData *)wdataP->privateData)->caret = 0;
	((Sample6_PrivateData *)wdataP->privateData)->length = 0;
	((Sample6_PrivateData *)wdataP->privateData)->numberOfSelectedLines = 0;
	((Sample6_PrivateData *)wdataP->privateData)->selBlocks = NULL;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickTime = 0;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickPoint.h = -50;
	((Sample6_PrivateData *)wdataP->privateData)->lastClickPoint.v = -50;
	
	// let's add 2 buttons
	Rect button1Rect = {392, 490, 410, 610};
	ControlHandle button1 = NewControl(aWind, &button1Rect, "\pShow Clipboard", true, 0, 0, 1, pushButProc, 1);
	Rect button2Rect = {392, 430, 410, 480};
	ControlHandle button2 = NewControl(aWind, &button2Rect, "\pCopy", true, 0, 0, 1, pushButProc, 2);
	
	return status;
	}

OSErr CreateClipboardWindow()
	{
	OSStatus status = noErr;
	Rect boundsRect = {150, 110, 570, 730};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	gClipboardWindow = NewCWindow(NULL, &boundsRect, "\pClipboard", false, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some default text
	UniCharArrayPtr unicodeText;
	UniCharCount unicodeTextLength;
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kNothingUnicodeText, &unicodeText, &unicodeTextLength);
	
	// and put it in the scrap as unicode text
	ZeroScrap();
	ByteCount byteLength = unicodeTextLength * 2;
	Handle hUnicodeText = NewHandle(byteLength);
	HLock(hUnicodeText);
	BlockMoveData((Ptr)unicodeText, *hUnicodeText, byteLength);
	PutScrap(byteLength, 'utxt', *hUnicodeText);

	// and ASCII text
	char *ASCIIText;
	QuickAndDirtySetASCII_C_CharsFromUnicodeText(&ASCIIText, (UniCharArrayPtr)*hUnicodeText, unicodeTextLength);
	Handle hText = NewHandle(unicodeTextLength);
	HLock(hText);
	BlockMoveData((Ptr)ASCIIText, *hText, unicodeTextLength);
	PutScrap(unicodeTextLength, 'TEXT', *hText);

	DisposeHandle(hUnicodeText);
	DisposeHandle(hText);
	DisposePtr((Ptr)ASCIIText);
	
	// simulate a paste
	DoPaste();
	
	return status;
	}

const kMaximumNameLength = 256;

void DoCopy(WindowDataPtr wdataP)
	{
	OSStatus status = noErr;
	Sample6_PrivateData *priv = (Sample6_PrivateData *)wdataP->privateData;
	if (priv->length == 0)
		{
		// nothing to copy...
		SysBeep(1);
		return;
		}

	if (gClipboardWindow == NULL) CreateClipboardWindow();
	Boolean wasVisible = ((WindowPeek)gClipboardWindow)->visible;
	if (wasVisible) HideWindow(gClipboardWindow);

	// let's grab the selected text
	ZeroScrap();
	ByteCount byteLength = priv->length * 2;
	Handle hUnicodeText = NewHandle(byteLength);
	HLock(hUnicodeText);
	BlockMoveData((Ptr)((long)wdataP->theUnicodeText + priv->caret * 2), *hUnicodeText, byteLength);

	// let's create the 'ustl' handle.
	// just look at ATSUTextConversion.h to get the detail of the structure.
	//
	// REMEMBER, don't use PPC (long) alignment or else sizeof(ATSUVersionType) will return 2
	//           but be aligned as 4 in the structures...
	//
	Size bufSize = 2000, realSize = 0, currentStyleOffset;
	Handle ustl = NewHandle(bufSize);
	HLock(ustl);
	
	ATSUStyleStreamHeader *ssh = (ATSUStyleStreamHeader *)(*ustl);
	ssh->fVersion = 0;
	ssh->fTextLength = priv->length;
	ssh->fLayoutRunCount = 0; // in this sample code, we have only the default layout control
	ssh->fLayoutRunOffset = 0;
	ssh->fStyleRunOffset = kATSUStyleStreamHeaderSize;
	realSize = kATSUStyleStreamHeaderSize;

	currentStyleOffset = realSize;
	ATSUStyleRunStreamHeader *srsh = (ATSUStyleRunStreamHeader *)((long)*ustl + currentStyleOffset);
	
	// create a Style container
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
	
	// let's grab all the styles of the selected text
	ItemCount nbStyles = 0;
	UniCharArrayOffset here = 0, start;
	UniCharCount length;
	while (here < priv->length)
		{
		status = ATSUGetRunStyle(wdataP->textLayout, priv->caret+here, &tempS, &start, &length);
		if (status != noErr)
			{
			DebugStr("\p ATSUGetRunStyle failed");
			break;
			}
		if ((start + length) > (priv->caret + priv->length))
			length = priv->caret + priv->length - start;
		srsh->fRunLength = start + length - priv->caret - here;
		ByteCount dataLength = sizeof( ATSUVersionType ) + 3 * sizeof( ItemCount );	
		srsh->fRunStreamVersion = 0;
		srsh->fRunFeatureCount = 0; // in this sample code, we don't deal with font features.
		srsh->fRunVariationCount = 0; // in this sample code, we don't deal with font variations.

		// getting all attributes
		ItemCount attributeCount;
		// let's first know how many ...
		status = ATSUGetAllAttributes(tempS, NULL, 0, &attributeCount);
		if (status != noErr) DebugStr("\p ATSUGetAllAttributes first time failed");
		// ... to allocate appropriately
		ATSUAttributeInfo *attributeInfoArray = (ATSUAttributeInfo *)NewPtr(attributeCount * sizeof(ATSUAttributeInfo));
		// and get them
		status = ATSUGetAllAttributes(tempS, attributeInfoArray, attributeCount, &attributeCount);
		if (status != noErr) DebugStr("\p ATSUGetAllAttributes second time failed");
		
		srsh->fRunAttributeCount = attributeCount;
		realSize += sizeof(ATSUStyleRunStreamHeader);
		ATSUAttributeInfo *aih = (ATSUAttributeInfo *)((long)*ustl + realSize);
		
		ItemCount attr;
		for (attr = 0; attr < attributeCount; attr++)
			{
			ByteCount attrLength = attributeInfoArray[attr].fValueSize;
			Ptr p = NewPtr(attrLength);
			status = ATSUGetAttribute(tempS, attributeInfoArray[attr].fTag, attrLength, p, &attrLength);
			if (status != noErr) DebugStr("\p ATSUGetAttribute failed");
			
			// if we get the kATSUFontTag tag, then we issue the font name rather than the font ID
			if (attributeInfoArray[attr].fTag == kATSUFontTag)
				{
				unsigned char fontName[ kMaximumNameLength ];
				ByteCount fontNameLength = 0;
				ATSUFontID theFontID = *((ATSUFontID *) p);
				status = ATSUFindFontName( theFontID, 
									3,	// kFontUniqueName,		(constant missing from SFNTTypes.h)
									-1, // kFontNoPlatform, 	(constant missing from SFNTTypes.h)
									-1, // kFontNoScript, 		(constant missing from SFNTTypes.h)
									-1, // kFontNoLanguage, 	(constant missing from SFNTTypes.h)
									kMaximumNameLength, 
									Ptr( fontName ), 
									&fontNameLength, 
									NULL );
				if (status != noErr) DebugStr("\p ATSUFindFontName failed");
				fontName[ fontNameLength ] = '\0';
				DisposePtr(p);
				attrLength = fontNameLength;
				p = NewPtr(attrLength);
				BlockMoveData((Ptr)fontName, p, attrLength);
				}
			
			// using the 300 margin, we only deal with size modification here
			if (realSize + attrLength > bufSize - 300)
				{
				bufSize += 2000;
				Handle h = NewHandle(bufSize);
				HLock(h);
				BlockMoveData(*ustl, *h, realSize);
				DisposeHandle(ustl);
				ustl = h;
				aih = (ATSUAttributeInfo *)((long)*ustl + realSize);
				}
			
			aih->fTag = attributeInfoArray[attr].fTag;
			aih->fValueSize = attrLength;
			BlockMoveData(p, (Ptr)((long)aih + sizeof(ATSUAttributeInfo)), attrLength);
			
			dataLength += sizeof(ATSUAttributeInfo) + attrLength;
			realSize += sizeof(ATSUAttributeInfo) + attrLength;
			aih = (ATSUAttributeInfo *)((long)*ustl + realSize);
			DisposePtr(p);
			}

		srsh = (ATSUStyleRunStreamHeader *)((long)*ustl + currentStyleOffset);
		srsh->fRunDataSize = dataLength;
		currentStyleOffset = realSize;
		srsh = (ATSUStyleRunStreamHeader *)((long)*ustl + currentStyleOffset);
		
		nbStyles++;
		here = start + length - priv->caret;
		}

	// let's adjust the size
	ssh = (ATSUStyleStreamHeader *)(*ustl);
	ssh->fStyleRunCount = nbStyles;
	HUnlock(ustl);
	SetHandleSize(ustl, realSize);
	HLock(ustl);

	// pre-allocating the handles
	Handle hTEText, hTEStyles;
	hTEText = NewHandle(0);
	hTEStyles = NewHandle(0);

	// converting...
	status = ATSUConvertToTEStyledText(hUnicodeText, ustl, hTEText, hTEStyles);
	if (status != noErr) DebugStr("\p ATSUConvertToTEStyledText failed");
	
	HLock(hTEText);
	HLock(hTEStyles);

	// putting in scrap
	PutScrap(byteLength, 'utxt', *hUnicodeText);
	PutScrap(GetHandleSize((Handle)ustl), 'ustl', *ustl);
	PutScrap(GetHandleSize((Handle)hTEText), 'TEXT', *hTEText);
	PutScrap(GetHandleSize((Handle)hTEStyles), 'styl', *hTEStyles);

	// and cleaning up.
	DisposeHandle(hUnicodeText);
	DisposeHandle(ustl);
	DisposeHandle(hTEText);
	DisposeHandle(hTEStyles);

	DoPaste();

	if (wasVisible) DoShowClipboard();

	return;
	}

void DoPaste()
	{
	OSStatus status = noErr;
	ATSUStyle tempS;

	// first the memory cleanup
	WindowDataPtr clipDataP = GetWindowData(gClipboardWindow);
	WindowDataPtr justTheUnicodeText = (WindowDataPtr)clipDataP->privateData;
	if (justTheUnicodeText != NULL)
		{
		SimpleQDText *sqdt = (SimpleQDText *)justTheUnicodeText->privateData;
		if (sqdt != NULL)
			{
			if (sqdt->justTheText != NULL) TEDispose(sqdt->justTheText);
			if (sqdt->textWithStyl != NULL) TEDispose(sqdt->textWithStyl);
			}
		DisposeWindowDataPtr(justTheUnicodeText);
		}
	DisposeWindowDataPtr(clipDataP);
	clipDataP = DefaultWindowDataPtr();
	SetWRefCon(gClipboardWindow, (long)clipDataP);

	// getting the scrap
	LoadScrap();
	long offset, byteLength = GetScrap(NULL, 'utxt', &offset);
	if (byteLength <= 0) return;
	Handle hUnicodeText = NewHandle(0);
	GetScrap(hUnicodeText, 'utxt', &offset);
	
	// let's grab the text from the scrap
	clipDataP->uTextLength = byteLength / 2;
	clipDataP->theUnicodeText = (UniCharArrayPtr)NewPtr(byteLength);
	HLock(hUnicodeText);
	BlockMoveData(*hUnicodeText, (Ptr)clipDataP->theUnicodeText, byteLength);
	
	// let's grab the styles from the scrap
	long styleLength = GetScrap(NULL, 'ustl', &offset);
	if (styleLength <= 0)
		{
		// no 'ustl' resource or something went wrong...
		// going back to the default...
		
		// only 1 style thus only 1 run
		clipDataP->numberOfRuns = 1;
		clipDataP->runLengths = (UniCharCount *)NewPtr(clipDataP->numberOfRuns * sizeof(UniCharCount *));
		clipDataP->runLengths[0] = clipDataP->uTextLength;

		// and it's the default style
		clipDataP->styles = (ATSUStyle *)NewPtr(clipDataP->numberOfRuns * sizeof(ATSUStyle *));
		status = ATSUCreateStyle(&tempS);
		if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
		clipDataP->styles[0] = tempS;
		}
	else
		{
		// getting the ustl resource
		Handle ustl = NewHandle(0);
		GetScrap(ustl, 'ustl', &offset);
		HLock(ustl);

		// allocating the arrays
		ATSUStyleStreamHeader *ssh = (ATSUStyleStreamHeader *)(*ustl);
		clipDataP->numberOfRuns = ssh->fStyleRunCount;
		clipDataP->runLengths = (UniCharCount *)NewPtr(clipDataP->numberOfRuns * sizeof(UniCharCount *));
		clipDataP->styles = (ATSUStyle *)NewPtr(clipDataP->numberOfRuns * sizeof(ATSUStyle *));

		// getting back all the styles
		ItemCount i;
		Ptr pos = (Ptr)((long)(*ustl) + ssh->fStyleRunOffset);
		for (i = 0; i < clipDataP->numberOfRuns; i++)
			{
			status = ATSUCreateStyle(&tempS);
			if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
			
			ATSUStyleRunStreamHeader *srsh = (ATSUStyleRunStreamHeader *)pos;
			clipDataP->runLengths[i] = srsh->fRunLength;
			ByteCount dataLength = sizeof(UniCharCount) + sizeof(ByteCount) + srsh->fRunDataSize;
			Ptr pos2 = pos + sizeof(ATSUStyleRunStreamHeader);
			pos += dataLength;
			
			ItemCount attr;
			for (attr = 0; attr < srsh->fRunAttributeCount; attr++)
				{
				ATSUAttributeInfo *aih = (ATSUAttributeInfo *)pos2;
				ATSUAttributeTag theTag = aih->fTag;
				ByteCount theLength = aih->fValueSize;
				Ptr thePtr = pos2 + sizeof(ATSUAttributeInfo);
				pos2 += sizeof(ATSUAttributeInfo) + theLength;
				
				// if we get the kATSUFontTag tag, then we get the font name rather than the font ID
				if (theTag == kATSUFontTag)
					{
					char fontName[ kMaximumNameLength ];
					BlockMoveData( thePtr, fontName, theLength );
					fontName[ theLength ] = '\0';
					ATSUFontID theFontID;
					status = ATSUFindFontFromName(	fontName,
															theLength,
															3,	// kFontUniqueName,		(constant missing from SFNTTypes.h)
															-1, // kFontNoPlatform, 	(constant missing from SFNTTypes.h)
															-1, // kFontNoScript, 		(constant missing from SFNTTypes.h)
															-1, // kFontNoLanguage, 	(constant missing from SFNTTypes.h)
										 					&theFontID );
					if ( status == noErr && theFontID != kATSUInvalidFontID )
						{
						theLength = sizeof( ATSUFontID );
						thePtr = (Ptr)&theFontID;
						}
					}
				// setting the attributes one by one.
				if ( status == noErr )
					{
					status = ATSUSetAttributes(tempS, 1, &theTag, &theLength, &thePtr);				
					if (status != noErr) DebugStr("\p ATSUSetAttributes failed");
					}
				}
			
			clipDataP->styles[i] = tempS;
			}
		DisposeHandle(ustl);
		}

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(clipDataP->theUnicodeText, 0, clipDataP->uTextLength, clipDataP->uTextLength, clipDataP->numberOfRuns, clipDataP->runLengths, clipDataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	clipDataP->textLayout = tempTL;
	
	// now, let's break the text in pieces
	BreakTheTextInLines(gClipboardWindow, clipDataP);

	// another time just for the unicode text
	justTheUnicodeText = (WindowDataPtr)NewPtr(sizeof(WindowData));

	// let's grab the text
	justTheUnicodeText->uTextLength = byteLength / 2;
	justTheUnicodeText->theUnicodeText = (UniCharArrayPtr)NewPtr(byteLength);
	BlockMoveData(*hUnicodeText, (Ptr)justTheUnicodeText->theUnicodeText, byteLength);
	DisposeHandle(hUnicodeText);
	
	// only 1 style thus only 1 run
	justTheUnicodeText->numberOfRuns = 1;
	justTheUnicodeText->runLengths = (UniCharCount *)NewPtr(justTheUnicodeText->numberOfRuns * sizeof(UniCharCount *));
	justTheUnicodeText->runLengths[0] = justTheUnicodeText->uTextLength;

	// and it's the default style
	justTheUnicodeText->styles = (ATSUStyle *)NewPtr(justTheUnicodeText->numberOfRuns * sizeof(ATSUStyle *));
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");
	justTheUnicodeText->styles[0] = tempS;

	// and we create the text layout
	status = ATSUCreateTextLayoutWithTextPtr(justTheUnicodeText->theUnicodeText, 0, justTheUnicodeText->uTextLength, justTheUnicodeText->uTextLength, justTheUnicodeText->numberOfRuns, justTheUnicodeText->runLengths, justTheUnicodeText->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	justTheUnicodeText->textLayout = tempTL;
	
	// now, let's break the text in pieces
	BreakTheTextInLines(gClipboardWindow, justTheUnicodeText);
	
	justTheUnicodeText->privateData = (void *)NewPtr(sizeof(SimpleQDText));
	
	// let's fill the TEHandles
	TEHandle aTE;
	GrafPtr savePort;
	GetPort(&savePort);
	SetPort(gClipboardWindow);
	
	TextFont(1);
	TextSize(9);
	
	TEFromScrap();
	
	Rect ra = r1;
	InsetRect(&ra, 2, 2);
	aTE = TEStyleNew(&ra, &ra);
	// just a TEPaste for the simple ASCII text
	TEPaste(aTE);
	((SimpleQDText *)(justTheUnicodeText->privateData))->justTheText = aTE;
	
	Rect rb = r2;
	InsetRect(&rb, 2, 2);
	aTE = TEStyleNew(&rb, &rb);
	// a TEStylePaste to also get the styles (as best as we can...)
	TEStylePaste(aTE);
	((SimpleQDText *)(justTheUnicodeText->privateData))->textWithStyl = aTE;

	SetPort(savePort);

	clipDataP->privateData = (void *)justTheUnicodeText;
	
	// to identify the window and validate all fields
	clipDataP->sampleKind = 100;
	}

void UpdateClipboard()
	{
	RgnHandle saveClip;
	saveClip = NewRgn();
	GetClip(saveClip);

	WindowDataPtr wdataP = GetWindowData(gClipboardWindow);
	if (wdataP->sampleKind == 100)
		{
		// all fields are valid
		TEHandle aTE;
		WindowDataPtr justTheUnicodeText = (WindowDataPtr)wdataP->privateData;
	
		TextFont(1);
		TextSize(9);

		// let's update just the ASCII text first
		Rect rb = r1;
		InsetRect(&rb, 2, 2);
		ClipRect(&rb);
		aTE = ((SimpleQDText *)(justTheUnicodeText->privateData))->justTheText;
		TEUpdate(&rb, aTE);
		
		// then TextEdit with Style
		Rect ra = r2;
		InsetRect(&ra, 2, 2);
		ClipRect(&ra);
		aTE = ((SimpleQDText *)(justTheUnicodeText->privateData))->textWithStyl;
		TEUpdate(&ra, aTE);

		TextFont(1);
		TextSize(9);

		// then just the Unicode Text in the single default style
		ClipRect(&r3);
			{
			ItemCount i;
			ATSUTextMeasurement x = ff(r3.left + 1);
			ATSUTextMeasurement y = ff(r3.top + 1) + justTheUnicodeText->lineHeight;
			UniCharArrayOffset lineStart = 0, lineEnd;
			for (i = 0; i < justTheUnicodeText->numberOfLines; i++)
				{
				lineEnd = justTheUnicodeText->endOfLines[i];
				OSStatus status = ATSUDrawText(justTheUnicodeText->textLayout, lineStart, lineEnd-lineStart, x, y);
				if (status != noErr) DebugStr("\p ATSUDrawText failed");
				y += justTheUnicodeText->lineHeight;
				lineStart = lineEnd;
				}
			}
		
		// and finally, the full Unicode Text and Styles
		ClipRect(&r4);
			{
			ItemCount i;
			ATSUTextMeasurement x = ff(r4.left + 1);
			ATSUTextMeasurement y = ff(r4.top + 1) + wdataP->lineHeight;
			UniCharArrayOffset lineStart = 0, lineEnd;
			for (i = 0; i < wdataP->numberOfLines; i++)
				{
				lineEnd = wdataP->endOfLines[i];
				OSStatus status = ATSUDrawText(wdataP->textLayout, lineStart, lineEnd-lineStart, x, y);
				if (status != noErr) DebugStr("\p ATSUDrawText failed");
				y += wdataP->lineHeight;
				lineStart = lineEnd;
				}
			}
		}
	
	SetClip(saveClip);
	DisposeRgn(saveClip);

	// some frames to be nice
	FrameRect(&r1);
	FrameRect(&r2);
	FrameRect(&r3);
	FrameRect(&r4);
	
	TextFont(1);
	TextSize(9);
	
	// and reminder labels
	MoveTo(r1.left, r1.bottom + 8); DrawString("\pTEXT");
	MoveTo(r2.left, r2.bottom + 8); DrawString("\pTEXT + styl");
	MoveTo(r3.left, r3.bottom + 8); DrawString("\putxt");
	MoveTo(r4.left, r4.bottom + 8); DrawString("\putxt + ustl");
	}

void DoShowClipboard()
	{
	if (gClipboardWindow == NULL) CreateClipboardWindow();
	ShowWindow(gClipboardWindow);
	SelectWindow(gClipboardWindow);
	}
