/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SampleWindow_9.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Implements the Window from menu item 9 of 'Demos' menu:
#	
#		Font Menu
#
------------------------------------------------------------------------------*/

#include <ATSUnicode.h>

#include "MoreATSUnicode.h"

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "SampleWindow_4.h"
#include "SampleWindow_9.h"

OSErr ShowFontMenu()
	{
	OSStatus status = noErr;
	WindowPtr aWind;
	Rect boundsRect = {50, 10, 470, 630};
	
	WindowDataPtr wdataP = DefaultWindowDataPtr();
	aWind = NewCWindow(NULL, &boundsRect, "\pFont Menu", true, noGrowDocProc, (WindowPtr)-1L, true, (long)wdataP);
	
	// let's use some very long text
	QuickAndDirtySetUnicodeTextFromASCII_C_Chars(kSomeVeryLongUnicodeText, &(wdataP->theUnicodeText), &(wdataP->uTextLength));

	// only 1 style thus only 1 run
	wdataP->numberOfRuns = 1;
	wdataP->runLengths = (UniCharCount *)NewPtr(wdataP->numberOfRuns * sizeof(UniCharCount *));
	wdataP->runLengths[0] = wdataP->uTextLength;

	// and it's the default style
	wdataP->styles = (ATSUStyle *)NewPtr(wdataP->numberOfRuns * sizeof(ATSUStyle *));
	ATSUStyle tempS;
	status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	TryToSetFontTo(tempS, "Chicago");

	// let's set the font size
	status = atsuSetSize(tempS, ff(14));
	if (status != noErr) DebugStr("\p atsuSetSize failed");

	wdataP->styles[0] = tempS;

	// and we create the text layout
	ATSUTextLayout tempTL;
	status = ATSUCreateTextLayoutWithTextPtr(wdataP->theUnicodeText, 0, wdataP->uTextLength, wdataP->uTextLength, wdataP->numberOfRuns, wdataP->runLengths, wdataP->styles, &tempTL);
	if (status != noErr) DebugStr("\p ATSUCreateTextLayoutWithTextPtr failed");
	wdataP->textLayout = tempTL;
	
	// to be drawn at
	wdataP->xLocation = ff(10);
	wdataP->yLocation = ff(20);
	
	// now, let's break the text in pieces
	BreakTheTextInLines(aWind, wdataP);
	
	// to identify the window when we click on the button (see below)
	wdataP->sampleKind = 9;
	
	// let's add the menu button
	void *menuLookupCookie;
	short submenuCount;
	short menuID = UniqueMenuId();
	MenuHandle theMenu = NewMenu(menuID, "\pFonts");
	status = atsuAppendFontMenu(theMenu, menuID+1, &submenuCount, &menuLookupCookie);
	if (status != noErr) DebugStr("\p atsuAppendFontMenu failed");
	
	Rect buttonRect = {392, 540, 410, 610};
	InsertMenu(theMenu, -1);
	ControlHandle button = NewControl(aWind, &buttonRect, "\pFonts", true, menuID,
							kControlBehaviorCommandMenu + kControlContentTextOnly, 0,
							kControlBevelButtonNormalBevelProc, (long)wdataP);

	// let's keep some stuff around...
	wdataP->privateData = (void *)NewPtr(sizeof(Sample9_PrivateData));
	((Sample9_PrivateData *)wdataP->privateData)->menuID = menuID;
	((Sample9_PrivateData *)wdataP->privateData)->menuLookupCookie = menuLookupCookie;
	((Sample9_PrivateData *)wdataP->privateData)->theMenu = theMenu;
	((Sample9_PrivateData *)wdataP->privateData)->button = button;
	
	return status;
	}

void ChangeTheFont(WindowDataPtr wdataP)
	{
	Sample9_PrivateData *priv = (Sample9_PrivateData *)wdataP->privateData;
	
	// which value?
	SInt16 value;
	GetBevelButtonMenuValue(priv->button, &value);
	
	// in which menu (or submenu...)
	Size outActualSize;
	short aMenuID;
	GetControlData(priv->button, 0, kControlBevelButtonLastMenuTag, sizeof(SInt16), (Ptr)&aMenuID, &outActualSize);
	
	ATSUFontID theFontID = atsuGetFontForFontMenuItem(aMenuID, value, priv->menuLookupCookie);
	if (theFontID == kATSUInvalidFontID)
		{
		DebugStr("\p atsuGetFontForFontMenuItem failed");
		return;
		}
	
	GrafPtr currentPort;
	GetPort(&currentPort);
	
	// let's grab the unique style:
	
	// create a Style container
	ATSUStyle tempS;
	OSStatus status = ATSUCreateStyle(&tempS);
	if (status != noErr) DebugStr("\p ATSUCreateStyle failed");

	// grab the continuous style of our text layout
	status = ATSUGetContinuousAttributes(wdataP->textLayout, 0, wdataP->uTextLength, tempS);
	if (status != noErr) DebugStr("\p ATSUGetContinuousAttributes failed");

	// set the new font in the style
	status = atsuSetFont(tempS, theFontID);
	if (status != noErr) DebugStr("\p atsuSetFont failed");
	
	// apply the style to the text layout
	status = ATSUSetRunStyle(wdataP->textLayout, tempS, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUSetRunStyle failed");
	
	// clear all the current soft breaks
	status = ATSUClearSoftLineBreaks(wdataP->textLayout, 0, wdataP->uTextLength);
	if (status != noErr) DebugStr("\p ATSUClearSoftLineBreaks failed");
	
	// dispose of the current endOfLines array
	if (wdataP->endOfLines != NULL) DisposePtr((Ptr)wdataP->endOfLines);
	
	// and let's break the text in pieces again
	BreakTheTextInLines(currentPort, wdataP);

	// ask for an update...
	EraseRect(&currentPort->portRect);
	InvalRect(&currentPort->portRect);
	}

void DisposeSample9Window(WindowDataPtr wdataP)
	{
	Sample9_PrivateData *priv = (Sample9_PrivateData *)wdataP->privateData;
	DeleteMenu(priv->menuID);
	if (priv->theMenu != nil) DisposeMenu(priv->theMenu);
	atsuDisposeFontMenuLookupCookie(priv->menuLookupCookie);
	}

short UniqueMenuId()
	{
	short i, result = -1;
	for (i = 20000; result == -1; i+=100) if (GetMenuHandle(i) == nil) result = i;
	return result;
	}
