/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	SomeUnicodeStuff.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	Since the purpose of the TypeServicesForUnicode Sample Application is to
#	demonstrate the use of Apple Text Services For Unicode Imaging technology
#	introduced in Mac OS 8.5, and since we need Unicode text to display,
#	the purpose of this header and source code is to deal with this Unicode text.
#
------------------------------------------------------------------------------*/

#include <string.h>
#include <TextEncodingConverter.h>

#include "SomeUnicodeStuff.h"

void QuickAndDirtySetUnicodeTextFromASCII_C_Chars(long whichText, UniCharArrayPtr *ucap, UniCharCount *ucc)
	{
	char *someText;
	switch (whichText)
		{
		case kHelloWorldUnicodeText:
			someText = "Hello World!";
			break;

		case kSomeLongerUnicodeText:
			someText = "This is a much longer text so that we can apply some styles!";
			//         "123456789012345678901234567890123456789012345678901234567890"
			break;

		case kSomeVeryLongUnicodeText:
			{
			// let's use some very long text
			// notice the single \n at the end of sentence1 and the double \n at the end of sentence5

			//                 "                                                                                                    1                                                                                                   2                                       ";
			//                 "0         1         2         3         4         5         6         7         8         9         0         1         2         3         4         5         6         7         8         9         0         1         2         3         ";
			//                 "012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789";
			char sentence1[] = "This chapter provides an overview of Apple Type Services for Unicode Imaging (ATSUI), an enhancement of QuickDraw that enables your application to do direct Unicode drawing with support for full multilingual text and advanced typography.\n";
			char sentence2[] = "The basic intent of ATSUI is to provide the ability to use Unicode text directly in a QuickDraw-based application. ";
			char sentence3[] = "Unlike QuickDraw GX, ATSUI can only be used to draw Unicode. ";
			char sentence4[] = "This simplifies the interface and implementation and provides for the greatest flexibility at the least expense. ";
			char sentence5[] = "ATSUI also provides more extensive ancillary support for text-drawing, such as word- and line-breaking functions.\n\n";
			char sentence6[] = "In addition to providing support for low-end Unicode drawing, ATSUI also provides access to advanced typography features. ";
			char sentence7[] = "It enables font designers to build stylistic sophistication like swash variants and arbitrary ligatures.";
			char bigbuffer[3000];
			strcpy(bigbuffer, sentence1);
			strcat(bigbuffer, sentence2);
			strcat(bigbuffer, sentence3);
			strcat(bigbuffer, sentence4);
			strcat(bigbuffer, sentence5);
			strcat(bigbuffer, sentence6);
			strcat(bigbuffer, sentence7);
			someText = bigbuffer;
			}
			break;

		case kSomeMixedUnicodeText:
			{
			// sentence2 will contain some Right-to-left text
			char sentence1[] = "This chapter provides an overview of Apple Type Services for Unicode Imaging (ATSUI), an enhancement of QuickDraw that enables your application to do direct Unicode drawing with support for full multilingual text and advanced typography.\n";
			char sentence2[] = "The ~basic intent of ATSUI is to provide~ the ability to use Unicode text directly in a QuickDraw-based application. ";
			char sentence3[] = "Unlike QuickDraw GX, ATSUI can only be used to draw Unicode. ";
			char sentence4[] = "This simplifies the interface and implementation and provides for the greatest flexibility at the least expense. ";
			char sentence5[] = "ATSUI also provides more extensive ancillary support for text-drawing, such as word- and line-breaking functions.\n";
			char sentence6[] = "In addition to providing support for low-end Unicode drawing, ATSUI also provides access to advanced typography features. ";
			char sentence7[] = "It enables font designers to build stylistic sophistication like swash variants and arbitrary ligatures.\n\n";
			char sentence8[] = "The -- and ++ buttons decrease and increase the text sizes. The Home button moves the caret to the start of the text without any selection.";
			char sentence9[] = "The Move to previous word and Move to next word buttons move the left-edge of the 30-characters selection by one Unicode Word. The Hilight next 30 button moves the selection by 30 characters.";
			char bigbuffer[3000];
			strcpy(bigbuffer, sentence1);
			strcat(bigbuffer, sentence2);
			strcat(bigbuffer, sentence3);
			strcat(bigbuffer, sentence4);
			strcat(bigbuffer, sentence5);
			strcat(bigbuffer, sentence6);
			strcat(bigbuffer, sentence7);
			strcat(bigbuffer, sentence8);
			strcat(bigbuffer, sentence9);
			someText = bigbuffer;
			}
			break;

		case kSomeOtherMixedUnicodeText:
			{
			// sentence 8 contains runs of blanks and of punctuation to test the double-clicking
			char sentence1[] = "This chapter provides an overview of Apple Type Services for Unicode Imaging (ATSUI), an enhancement of QuickDraw that enables your application to do direct Unicode drawing with support for full multilingual text and advanced typography.\n";
			char sentence2[] = "The ~basic intent of ATSUI is to provide~ the ability to use Unicode text directly in a QuickDraw-based application. ";
			char sentence3[] = "Unlike QuickDraw GX, ATSUI can only be used to draw Unicode. ";
			char sentence4[] = "This simplifies the interface and implementation and provides for the greatest flexibility at the least expense. ";
			char sentence5[] = "ATSUI also provides more extensive ancillary support for text-drawing, such as word- and line-breaking functions.\n\n";
			char sentence6[] = "In addition to providing support for low-end Unicode drawing, ATSUI also provides access to advanced typography features. ";
			char sentence7[] = "It enables font designers to build stylistic sophistication like swash variants and arbitrary ligatures.\n\n";
			char sentence8[] = "This last sentence is there to [hit-]test the double-clicking with runs of blanks:     or of punctuation ,.;:.";
			char bigbuffer[3000];
			strcpy(bigbuffer, sentence1);
			strcat(bigbuffer, sentence2);
			strcat(bigbuffer, sentence3);
			strcat(bigbuffer, sentence4);
			strcat(bigbuffer, sentence5);
			strcat(bigbuffer, sentence6);
			strcat(bigbuffer, sentence7);
			strcat(bigbuffer, sentence8);
			someText = bigbuffer;
			}
			break;

		case kNothingUnicodeText:
			someText = "Nothing in Clipboard!";
			break;

		case kSomeMixedAgainUnicodeText:
			{
			char sentence1[] = "This chapter provides an overview of Apple Type Services for Unicode Imaging (ATSUI), an enhancement of QuickDraw that enables your application to do direct Unicode drawing with support for full multilingual text and advanced typography.\n";
			char sentence2[] = "The ~basic intent of ATSUI is to provide~ the ability to use Unicode text directly in a QuickDraw-based application. ";
			char sentence3[] = "Unlike QuickDraw GX, ATSUI can only be used to draw Unicode. ";
			char sentence4[] = "This simplifies the interface and implementation and provides for the greatest flexibility at the least expense. ";
			char sentence5[] = "ATSUI also provides more extensive ancillary support for text-drawing, such as word- and line-breaking functions.\n\n";
			char sentence6[] = "In addition to providing support for low-end Unicode drawing, ATSUI also provides access to advanced typography features. ";
			char sentence7[] = "It enables font designers to build stylistic sophistication like swash variants and arbitrary ligatures.";
			char bigbuffer[3000];
			strcpy(bigbuffer, sentence1);
			strcat(bigbuffer, sentence2);
			strcat(bigbuffer, sentence3);
			strcat(bigbuffer, sentence4);
			strcat(bigbuffer, sentence5);
			strcat(bigbuffer, sentence6);
			strcat(bigbuffer, sentence7);
			someText = bigbuffer;
			}
			break;
		}
	
	TECObjectRef ec;
	OSStatus status = TECCreateConverter(&ec, kTextEncodingMacRoman, kTextEncodingUnicodeDefault);
	if (status != noErr) DebugStr("\p TECCreateConverter failed");
	ByteCount ail, aol, iLen = strlen(someText), oLen = 2 * iLen;
	Ptr buffer = NewPtr(oLen);
	status = TECConvertText(ec, (ConstTextPtr)someText, iLen, &ail, (TextPtr)buffer, oLen, &aol);
	if (status != noErr) DebugStr("\p TECConvertText failed");
	status = TECDisposeConverter(ec);
	if (status != noErr) DebugStr("\p TECDisposeConverter failed");
	*ucap = (UniCharArrayPtr)NewPtr(aol);
	BlockMove(buffer, (*ucap), aol);
	DisposePtr(buffer);
	*ucc = aol / 2;

	switch (whichText)
		{
		case kSomeMixedUnicodeText:
		case kSomeOtherMixedUnicodeText:
		case kSomeMixedAgainUnicodeText:
			(*ucap)[243] = 0x202E; // Right-To-Left Override in Unicode.
			(*ucap)[279] = 0x202C; // Pop Directional Formatting in Unicode.
			break;
		}
	}

void QuickAndDirtySetASCII_C_CharsFromUnicodeText(char **text, UniCharArrayPtr ucap, UniCharCount ucc)
	{
	ByteCount ail, aol, bigLength = 2 * ucc;
	Ptr buffer = NewPtr(bigLength);
	TECObjectRef ec;
	OSStatus status = TECCreateConverter(&ec, kTextEncodingUnicodeDefault, kTextEncodingMacRoman);
	if (status != noErr) DebugStr("\p TECCreateConverter failed");
	status = TECConvertText(ec, (ConstTextPtr)ucap, bigLength, &ail, (TextPtr)buffer, bigLength, &aol);
	if (status != noErr) DebugStr("\p TECConvertText failed");
	status = TECDisposeConverter(ec);
	if (status != noErr) DebugStr("\p TECDisposeConverter failed");
	*text = (char *)NewPtr(aol);
	BlockMove(buffer, (*text), aol);
	DisposePtr(buffer);
	}
