/*------------------------------------------------------------------------------
#
#	Macintosh Developer Technical Support
#
#	Simple Apple Text Services For Unicode Imaging (ATSUI) Sample Application
#
#	TypeServicesForUnicode
#
#	TypeServicesForUnicode.cp	-	C++ Source
#
#	Copyright  1998 Apple Computer, Inc.
#	All rights reserved.
#	
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
#	Versions:	1.0					7/98
#
#	This is a very simple sample program that demonstrates how to use the new 
#	Apple Text Services For Unicode Imaging technology introduced in Mac OS 8.5.
#
------------------------------------------------------------------------------*/

#include <string.h>
#include <stdio.h>
#include <ATSUnicode.h>

#include "SomeUnicodeStuff.h"
#include "WindowUtilities.h"
#include "TypeServicesForUnicode.h"
#include "SampleWindow_1.h"
#include "SampleWindow_2.h"
#include "SampleWindow_3.h"
#include "SampleWindow_4.h"
#include "SampleWindow_5.h"
#include "SampleWindow_6.h"
#include "SampleWindow_7.h"
#include "SampleWindow_8.h"
#include "SampleWindow_9.h"

Boolean stop;
Boolean sawMemoryAlert;

int main(void)
{
	InitializeMac();
	if (!CanUseATSUI())
		{
		StopAlert(128, NULL);
		return 0;
		}
	InitializeAppl();
	
	do MainEventLoop(); while (stop == false);

	return 0;	
}

// 
//	Initialize the Toolbox
//

void InitializeMac()
{
	/* Initialize all the needed managers. */
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	FlushEvents(everyEvent,0);
	TEInit();
	InitDialogs(NULL);
	InitCursor();
	
	MaxApplZone();
}

// 
//	Initialize the Application
//

void InitializeAppl()
{
	Handle menuBar = GetNewMBar(128);
	SetMenuBar(menuBar);
	DrawMenuBar();
	DisposeHandle(menuBar);
	AppendResMenu(GetMenuHandle(1), 'DRVR');
	InstallAppleEventHandlers();
	stop = false;
	sawMemoryAlert = false;
}

// 
//	Initialize the standard Apple event handlers
//

void InstallAppleEventHandlers()
	{
	long result;
	OSErr err = Gestalt(gestaltAppleEventsAttr, &result);
	if (err == noErr)
		{	// we should care for the return values of AEInstallEventHandler but since it's just a sample...
		AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, NewAEEventHandlerProc(HandleOapp), 0, false);
		AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,   NewAEEventHandlerProc(HandleOdoc), 0, false);
		AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments,  NewAEEventHandlerProc(HandlePdoc), 0, false);
		AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerProc(HandleQuit), 0, false);
		}
	}

pascal OSErr HandleOapp (AEDescList *aevt, AEDescList *reply, long refCon)
	{
#pragma unused (aevt, reply, refCon)
	return noErr;
	}

pascal OSErr HandleOdoc (AEDescList *aevt, AEDescList *reply, long refCon)
	{
#pragma unused (aevt, reply, refCon)
	return errAEEventNotHandled;
	}

pascal OSErr HandlePdoc (AEDescList *aevt, AEDescList *reply, long refCon)
	{
#pragma unused (aevt, reply, refCon)
	return errAEEventNotHandled;
	}

pascal OSErr HandleQuit (AEDescList *aevt, AEDescList *reply, long refCon)
	{
#pragma unused (aevt, reply, refCon)
	stop = true;
	return noErr;
	}

Boolean CanUseATSUI()
	{
	long result;
	OSErr err = Gestalt(gestaltATSUVersion, &result);
	return (err == noErr);
	}

void MainEventLoop()
	{
	EventRecord theEvent;
	WindowPtr theWind;
	long sleep = 4;
	while (!stop)
		{
		if (!sawMemoryAlert && (FreeMem() < 100000))
			{
			StopAlert(130, NULL);
			sawMemoryAlert = true;
			}
		if (WaitNextEvent(everyEvent, &theEvent, sleep, 0L))
			switch (theEvent.what)
				{
				case mouseDown:
					switch (FindWindow(theEvent.where, &theWind))
						{
						case inMenuBar:
							AdjustMenus();
							DoMenuCommand(MenuSelect(theEvent.where));
							break;
						case inSysWindow:
							SystemClick(&theEvent, theWind);
							break;
						case inContent:
							if (theWind != FrontWindow())
								SelectWindow(theWind);
							else
								{
								Point thePoint = theEvent.where;
								ControlHandle theControl;
								GrafPtr savePort;
								GetPort(&savePort);
								SetPort(theWind);
								GlobalToLocal(&thePoint);
								short partCode = FindControl(thePoint, theWind, &theControl);
								if (partCode != 0)
									{
									short finalPartCode = TrackControl(theControl, thePoint, NULL);
									if (partCode == finalPartCode)
										{
										WindowDataPtr wdataP = GetWindowData(theWind);
										if (wdataP != NULL)
											{
											long whichControlRef = GetControlReference(theControl);
											switch (wdataP->sampleKind)
												{
												case 3: // only 1 button...
													RotateTheLine(wdataP);
													break;
												case 4: // Smaller or Bigger, 1 gives 1, 2 gives -1...
													SmallerBiggerFont(wdataP, ff(1 - 2 * (whichControlRef - 1)));
													break;
												case 5:
													if (whichControlRef <= 4)
														HilightAndOrMove(wdataP, GetControlReference(theControl));
													else
														{
														UnhilightCurrentSelection(wdataP);
														((Sample5_PrivateData *)wdataP->privateData)->withHilight = false;
														BetterSmallerBiggerFont(wdataP, ff(1 - 2 * (whichControlRef - 5)));
														}
													break;
												case 7:
													if (whichControlRef == 1)
														DoShowClipboard();
													else DoCopy(wdataP);
													break;
												}
											}
										}
									else if (finalPartCode == kControlMenuPart)
										{
										WindowDataPtr wdataP = GetWindowData(theWind);
										if (wdataP != NULL)
											{
											switch (wdataP->sampleKind)
												{
												case 9:
													ChangeTheFont(wdataP);
													break;
												}
											}
										}
									}
								else
									{
									WindowDataPtr wdataP = GetWindowData(theWind);
									if (wdataP != NULL)
										if ((wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
											ClickInSample6Window(wdataP, &theEvent);
									}
								SetPort(savePort);
								}
							break;
						case inDrag:
							DragWindow(theWind, theEvent.where, &qd.screenBits.bounds);
							break;
						case inGrow:
							break;
						case inGoAway:
							if (TrackGoAway(theWind, theEvent.where))
								if (theEvent.modifiers & optionKey)
									CloseAllWindows();
								else
									DoCloseWindow(theWind);
							break;
						case inZoomIn: case inZoomOut: default:
							break;
						}
					break;
				case keyDown: case autoKey:
					if (theEvent.modifiers & cmdKey)
						{
						AdjustMenus();
						DoMenuCommand(MenuKey(theEvent.message & charCodeMask));
						}
					else
						{
						// something...
						}
					break;
				case activateEvt:
					{
					theWind = (WindowPtr)theEvent.message;
					WindowDataPtr wdataP = GetWindowData(theWind);
					if (wdataP != NULL)
						if ((wdataP->sampleKind == 5) || (wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
							ActivateSample5Window(theWind, wdataP, theEvent.modifiers & activeFlag);
					}
					break;
				case updateEvt:
					DoUpdateWindow((WindowPtr)theEvent.message);
					break;
				case osEvt:
					if ((theEvent.message & osEvtMessageMask) == 0x01000000)
						{
						theWind = FrontWindow();
						WindowDataPtr wdataP = GetWindowData(theWind);
						if (theEvent.message & 1)
							{/* resume */
							sleep = 4;
							SetCursor(&qd.arrow);
							if (wdataP != NULL)
								if ((wdataP->sampleKind == 5) || (wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
									ActivateSample5Window(theWind, wdataP, true);
							}
						else
							{/* suspend  */
							sleep = 60;
							if (wdataP != NULL)
								if ((wdataP->sampleKind == 5) || (wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
									ActivateSample5Window(theWind, wdataP, false);
							}
						}
					break;
				case kHighLevelEvent:
					AEProcessAppleEvent(&theEvent);
					break;
				}
		}
	}

void AdjustMenus()
	{
	}

void DoMenuCommand(long menuResult)
	{
	switch(menuResult >> 16)
		{
		case 1:
			if ((menuResult & 0x0FFFF) == 1)
				{
				Alert(129, NULL);
				}
			else
				{
				Str255 daName;
				GetMenuItemText(GetMenuHandle(1), (menuResult & 0x0FFFF), daName);
				OpenDeskAcc(daName);
				}
			break;
		case 128:
			switch(menuResult & 0x0FFFF)
				{
				case 1:
					WindowPtr aWindow = FrontWindow();
					if (aWindow != NULL) DoCloseWindow(aWindow);
					break;
				case 3:
					stop = true;
					break;
				}
			break;
		case 129:
			switch(menuResult & 0x0FFFF)
				{
				case 1:
					NewSingleLineSingleStyleWindow();
					break;
				case 2:
					NewSingleLineMultipleStylesWindow();
					break;
				case 3:
					NewSingleVerticalLineMultipleStylesWindow();
					break;
				case 4:
					NewParagraphsWindow();
					break;
				case 5:
					NewHighlightingCaretsCursorMovementsWindow();
					break;
				case 6:
					NewHitTestingWindow();
					break;
				case 7:
					NewClipboardUsingWindow();
					break;
				case 8:
					ShowATSUITextBox();
					break;
				case 9:
					ShowFontMenu();
					break;
				}
			break;
		}
	HiliteMenu(0);
	}

void DoUpdateWindow(WindowPtr theWind)
	{
	GrafPtr savePort;
	GetPort(&savePort);
	SetPort(theWind);
	BeginUpdate(theWind);
	WindowDataPtr wdataP = GetWindowData(theWind);
	if (wdataP != NULL)
		{
		if (wdataP->sampleKind == 100)
			{
			// global clipboard window
			UpdateClipboard();
			}
		else if (wdataP->numberOfLines == 1)
			{
			// only 1 line to draw
			OSStatus status = ATSUDrawText(wdataP->textLayout, 0, wdataP->uTextLength, wdataP->xLocation, wdataP->yLocation);
			if (status != noErr) DebugStr("\p ATSUDrawText failed");
			}
		else
			{
			// the following code is only for multi-lines text (and horizontal only)
			ItemCount i;
			ATSUTextMeasurement y = wdataP->yLocation;
			UniCharArrayOffset lineStart = 0, lineEnd;
			for (i = 0; i < wdataP->numberOfLines; i++)
				{
				lineEnd = wdataP->endOfLines[i];
				OSStatus status = ATSUDrawText(wdataP->textLayout, lineStart, lineEnd-lineStart, wdataP->xLocation, y);
				if (status != noErr) DebugStr("\p ATSUDrawText failed");
				y += wdataP->lineHeight;
				lineStart = lineEnd;
				}
			}
		if ((wdataP->sampleKind == 5) || (wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
			UpdateSample5Window(theWind, wdataP);
		}
	UpdateControls(theWind, theWind->visRgn);
	EndUpdate(theWind);
	SetPort(savePort);
	}

void DoCloseWindow(WindowPtr theWind)
	{
	WindowDataPtr wdataP = GetWindowData(theWind);
	if (wdataP != NULL)
		{
		if (wdataP->sampleKind == 100)
			{
			// global clipboard window
			HideWindow(theWind);
			return;
			}
		if ((wdataP->sampleKind == 5) || (wdataP->sampleKind == 6) || (wdataP->sampleKind == 7))
			DisposeSample5Window(wdataP);
		if (wdataP->sampleKind == 9)
			DisposeSample9Window(wdataP);
		DisposeWindowDataPtr(wdataP);
		}
	DisposeWindow((WindowPtr)theWind);
	}

void RecursiveCloseWindow(WindowPeek theWind)
	{
	WindowPeek nextWind = theWind->nextWindow;
	if (nextWind) RecursiveCloseWindow(nextWind);
	DoCloseWindow((WindowPtr)theWind);
	}

void CloseAllWindows()
	{
	WindowPtr theWind = FrontWindow();
	if (theWind) RecursiveCloseWindow((WindowPeek)theWind);
	}

// some debug stuff

				/*{
				ItemCount nbFonts;
				OSStatus stat2 = ATSUFontCount(&nbFonts);
				ATSUFontID *fontIDArray = (ATSUFontID *)NewPtr(nbFonts * sizeof(ATSUFontID));
				stat2 = ATSUGetFontIDs(fontIDArray, nbFonts, &nbFonts);
				ItemCount i;
				for (i=0; i < nbFonts; i++)
					{
					ItemCount nbNames;
					stat2 = ATSUCountFontNames(fontIDArray[i], &nbNames);
					ItemCount j;
					char buffer[1000];
					ByteCount actualLength;
					FontNameCode name;
					FontPlatformCode platform;
					FontScriptCode script;
					FontLanguageCode language;
					for (j=0; j < 10; j++)
						{
						stat2 = ATSUGetIndFontName(fontIDArray[i], j, 1000, (Ptr)buffer, &actualLength, &name, &platform, &script, &language);
						buffer[actualLength] = 0;
						printf("%ld, %ld, %ld, %ld, %ld, %ld, '%s'\n", i, j, name, platform, script, language, buffer);
						}
					}
				}*/
